'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test tls server certificate verification options
'''

# Define default ATS
ts = Test.MakeATSProcess("ts", enable_tls=True)
server_foo = Test.MakeOriginServer(
    "server_foo",
    ssl=True,
    options={
        "--key": "{0}/signed-foo.key".format(Test.RunDirectory),
        "--cert": "{0}/signed-foo.pem".format(Test.RunDirectory)
    })
server_bar = Test.MakeOriginServer(
    "server_bar",
    ssl=True,
    options={
        "--key": "{0}/signed-bar.key".format(Test.RunDirectory),
        "--cert": "{0}/signed-bar.pem".format(Test.RunDirectory)
    })
server_wild = Test.MakeOriginServer(
    "server_wild",
    ssl=True,
    options={
        "--key": "{0}/signed-wild.key".format(Test.RunDirectory),
        "--cert": "{0}/signed-wild.pem".format(Test.RunDirectory)
    })
server = Test.MakeOriginServer("server", ssl=True)

request_foo_header = {"headers": "GET / HTTP/1.1\r\nHost: foo.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
request_bad_foo_header = {"headers": "GET / HTTP/1.1\r\nHost: badfoo.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
request_bar_header = {"headers": "GET / HTTP/1.1\r\nHost: bar.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
request_bad_bar_header = {"headers": "GET / HTTP/1.1\r\nHost: badbar.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server_foo.addResponse("sessionlog.json", request_foo_header, response_header)
server_foo.addResponse("sessionlog.json", request_bad_foo_header, response_header)
server_bar.addResponse("sessionlog.json", request_bar_header, response_header)
server_bar.addResponse("sessionlog.json", request_bad_bar_header, response_header)
server_wild.addResponse("sessionlog.json", request_bar_header, response_header)

# add ssl materials like key, certificates for the server
ts.addSSLfile("ssl/signed-foo.pem")
ts.addSSLfile("ssl/signed-foo.key")
ts.addSSLfile("ssl/signed-bar.pem")
ts.addSSLfile("ssl/signed-bar.key")
ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")
ts.addSSLfile("ssl/signer.pem")
ts.addSSLfile("ssl/signer.key")
ts.addSSLfile("ssl/signed-wild.key")
ts.addSSLfile("ssl/signed-wild.pem")

ts.Disk.remap_config.AddLine('map / https://127.0.0.1:{0}'.format(server.Variables.SSL_Port))
ts.Disk.remap_config.AddLine('map https://foo.com/ https://127.0.0.1:{0}'.format(server_foo.Variables.SSL_Port))
ts.Disk.remap_config.AddLine('map https://bad_foo.com/ https://127.0.0.1:{0}'.format(server_foo.Variables.SSL_Port))
ts.Disk.remap_config.AddLine('map https://bar.com/ https://127.0.0.1:{0}'.format(server_bar.Variables.SSL_Port))
ts.Disk.remap_config.AddLine('map https://bad_bar.com/ https://127.0.0.1:{0}'.format(server_bar.Variables.SSL_Port))
ts.Disk.remap_config.AddLine('map https://foo.wild.com/ https://127.0.0.1:{0}'.format(server_wild.Variables.SSL_Port))
ts.Disk.remap_config.AddLine('map https://foo_bar.wild.com/ https://127.0.0.1:{0}'.format(server_wild.Variables.SSL_Port))

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

# Case 1, global config policy=permissive properties=signature
#         override for foo.com policy=enforced properties=all
ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        # set global policy
        'proxy.config.ssl.client.verify.server.policy': 'PERMISSIVE',
        'proxy.config.ssl.client.verify.server.properties': 'SIGNATURE',
        'proxy.config.ssl.client.CA.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.client.CA.cert.filename': 'signer.pem',
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.url_remap.pristine_host_hdr': 1
    })

ts.Disk.sni_yaml.AddLines(
    [
        'sni:',
        '- fqdn: bar.com',
        '  verify_server_policy: ENFORCED',
        '  verify_server_properties: ALL',
        '- fqdn: "*.wild.com"',
        '  verify_server_policy: ENFORCED',
        '  verify_server_properties: ALL',
        '- fqdn: bad_bar.com',
        '  verify_server_policy: ENFORCED',
        '  verify_server_properties: ALL',
    ])

tr = Test.AddTestRun("Permissive-Test")
tr.Setup.Copy("ssl/signed-foo.key")
tr.Setup.Copy("ssl/signed-foo.pem")
tr.Setup.Copy("ssl/signed-bar.key")
tr.Setup.Copy("ssl/signed-bar.pem")
tr.Setup.Copy("ssl/signed-wild.pem")
tr.Setup.Copy("ssl/signed-wild.key")
tr.Processes.Default.Command = "curl -v -k -H \"host: foo.com\" https://127.0.0.1:{0}".format(ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.Processes.Default.StartBefore(server_foo)
tr.Processes.Default.StartBefore(server_bar)
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(server_wild)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.StillRunningAfter = server
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")

tr2 = Test.AddTestRun("Override-enforcing-Test")
tr2.Processes.Default.Command = "curl -v -k -H \"host: bar.com\"  https://127.0.0.1:{0}".format(ts.Variables.ssl_port)
tr2.ReturnCode = 0
tr2.StillRunningAfter = server
tr2.StillRunningAfter = ts
tr2.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")

tr3 = Test.AddTestRun("Override-enforcing-Test-fail-name-check")
tr3.Processes.Default.Command = "curl -v -k -H \"host: bad_bar.com\"  https://127.0.0.1:{0}".format(ts.Variables.ssl_port)
tr3.Processes.Default.Streams.stdout = Testers.ContainsExpression("Could Not Connect", "Curl attempt should have failed")
tr3.ReturnCode = 0
tr3.StillRunningAfter = server
tr3.StillRunningAfter = ts

tr4 = Test.AddTestRun("Exercise-wildcard-cert-name-check")
tr4.Processes.Default.Command = "curl -v -k -H \"host: foo.wild.com\"  https://127.0.0.1:{0}".format(ts.Variables.ssl_port)
tr4.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr4.ReturnCode = 0
tr4.StillRunningAfter = server
tr4.StillRunningAfter = ts

tr5 = Test.AddTestRun("Exercise-wildcard-cert-underscore-name-check")
tr5.Processes.Default.Command = "curl -v -k -H \"host: foo_bar.wild.com\"  https://127.0.0.1:{0}".format(ts.Variables.ssl_port)
tr5.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr5.ReturnCode = 0
tr5.StillRunningAfter = server
tr5.StillRunningAfter = ts

# Over riding the built in ERROR check since we expect tr3 to fail
ts.Disk.diags_log.Content = Testers.ExcludesExpression("verification failed", "Make sure the signatures didn't fail")
ts.Disk.diags_log.Content += Testers.ContainsExpression(
    r"WARNING: SNI \(bad_bar.com\) not in certificate", "Make sure bad_bar name checked failed.")



'''
Test the verify_global_plugin TrafficServer command.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Test that the TrafficServer verify_global_plugin command works as expected.
'''

process_counter = 0


def create_ts_process():
    """
    Create a unique ATS process with each call to this function.
    """
    global process_counter
    process_counter += 1

    ts = Test.MakeATSProcess("ts{counter}".format(counter=process_counter))

    # Ideally we would set the test run's Processes.Default to ts, but deep
    # copy of processes is not currently implemented in autest. Therefore we
    # replace the command which ts runs with a dummy command, and pull in
    # piecemeal the values from ts that we want into the test run.
    ts.Command = "sleep 100"
    # sleep will return -2 when autest kills it. We set the expectation for the
    # -2 return code here so the test doesn't fail because of this.
    ts.ReturnCode = -2
    # Clear the ready criteria because sleep is ready as soon as it is running.
    ts.Ready = None
    return ts


"""
TEST: verify_global_plugin should complain if an argument is not passed to it.
"""
tr = Test.AddTestRun("Verify the requirement of an argument")
ts = create_ts_process()
tr.Processes.Default.Env = ts.Env
tr.Processes.Default.Command = "traffic_server -C 'verify_global_plugin'"
tr.Processes.Default.ReturnCode = 1
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.Streams.stderr = Testers.ContainsExpression(
    "ERROR: verifying a plugin requires a plugin SO file path argument", "Should warn about the need for an SO file argument")
"""
TEST: verify_global_plugin should complain if the argument doesn't reference a shared
object file.
"""
tr = Test.AddTestRun("Verify the requirement of a file")
ts = create_ts_process()
tr.Processes.Default.Env = ts.Env
tr.Processes.Default.Command = \
    "traffic_server -C 'verify_global_plugin {filename}'".format(
        filename="/this/file/does/not/exist.so")
tr.Processes.Default.ReturnCode = 1
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.Streams.stderr = Testers.ContainsExpression(
    "ERROR: .*No such file or directory", "Should warn about the non-existent SO file argument")
"""
TEST: verify_global_plugin should complain if the shared object file doesn't
have the expected Plugin symbols.
"""
tr = Test.AddTestRun("Verify the requirement of our Plugin API.")
ts = create_ts_process()
Test.PrepareTestPlugin(os.path.join(Test.Variables.AtsTestPluginsDir, 'missing_ts_plugin_init.so'), ts)
tr.Processes.Default.Env = ts.Env
tr.Processes.Default.Command = \
    "traffic_server -C 'verify_global_plugin {filename}'".format(
        filename="${PROXY_CONFIG_PLUGIN_PLUGIN_DIR}/missing_ts_plugin_init.so")
tr.Processes.Default.ReturnCode = 1
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.Streams.stderr = Testers.ContainsExpression(
    "ERROR: .*unable to find TSPluginInit function in", "Should warn about the need for the TSPluginInit symbol")
ts.Disk.diags_log.Content = Testers.ContainsExpression("ERROR", "ERROR: .*unable to find TSPluginInit function in")
"""
TEST: Verify that passing a remap plugin produces a warning because
it doesn't have the global plugin symbols.
"""
tr = Test.AddTestRun("Verify a properly formed plugin works as expected.")
ts = create_ts_process()
Test.PrepareTestPlugin(os.path.join(Test.Variables.AtsTestPluginsDir, 'conf_remap_stripped.so'), ts)
tr.Processes.Default.Env = ts.Env
tr.Processes.Default.Command = \
    "traffic_server -C 'verify_global_plugin {filename}'".format(
        filename="${PROXY_CONFIG_PLUGIN_PLUGIN_DIR}/conf_remap_stripped.so")
tr.Processes.Default.ReturnCode = 1
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.Streams.stderr = Testers.ContainsExpression(
    "ERROR: .*unable to find TSPluginInit function in", "Should warn about the need for the TSPluginInit symbol")
ts.Disk.diags_log.Content = Testers.ContainsExpression("ERROR", "ERROR: .*unable to find TSPluginInit function in")
"""
TEST: The happy case: a global plugin shared object file is passed as an
argument that has the definition for the expected Plugin symbols.
"""
tr = Test.AddTestRun("Verify a properly formed plugin works as expected.")
ts = create_ts_process()
Test.PrepareTestPlugin(os.path.join(Test.Variables.AtsTestPluginsDir, 'ssl_hook_test.so'), ts)
tr.Processes.Default.Env = ts.Env
tr.Processes.Default.Command = \
    "traffic_server -C 'verify_global_plugin {filename}'".format(
        filename="${PROXY_CONFIG_PLUGIN_PLUGIN_DIR}/ssl_hook_test.so")
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.Streams.stderr = Testers.ContainsExpression(
    "NOTE: verifying plugin '.*' Success", "Verification should succeed")
"""
TEST: This is a regression test for a shared object file that doesn't have all
of the required symbols defined because of a malformed interaction between C
and C++ files.
"""
tr = Test.AddTestRun("Regression test for an undefined, mangled C++ symbol.")
ts = create_ts_process()
plugin_filename = 'missing_mangled_definition.so'
built_plugin_path = os.path.join(Test.Variables.AtsTestPluginsDir, plugin_filename)
ats_plugin_dir = ts.Env['PROXY_CONFIG_PLUGIN_PLUGIN_DIR']
ts.Setup.Copy(built_plugin_path, ats_plugin_dir)
tr.Processes.Default.Env = ts.Env
tr.Processes.Default.Command = \
    "traffic_server -C 'verify_global_plugin {plugin_path}'".format(
        plugin_path=os.path.join(ats_plugin_dir, plugin_filename))
tr.Processes.Default.ReturnCode = 1
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.Streams.stderr = Testers.ContainsExpression(
    "ERROR:.*unable to load", "Should log failure to load shared object")
ts.Disk.diags_log.Content = Testers.ContainsExpression("ERROR", "Should log failure to load shared object")



#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from typing import Optional

Test.Summary = 'Checks records.proxy.config.udp.poll_timeout'

Test.SkipUnless(Condition.HasATSFeature('TS_HAS_QUICHE'))


class TestPollTimeout:
    """Configure a test for poll_timeout."""

    ts_counter: int = 0

    def __init__(self, name: str, udp_poll_timeout_in: Optional[int] = None) -> None:
        """Initialize the test.

        :param name: The name of the test.
        :param udp_poll_timeout_in: Configuration value for proxy.config.udp.poll_timeout
        """
        self.name = name
        self.udp_poll_timeout_in = udp_poll_timeout_in
        self.expected_udp_poll_timeout = 100
        if udp_poll_timeout_in is not None:
            self.expected_udp_poll_timeout = udp_poll_timeout_in

    def _configure_traffic_server(self, tr: 'TestRun'):
        """Configure Traffic Server.

        :param tr: The TestRun object to associate the ts process with.
        """
        ts = tr.MakeATSProcess(f"ts-{TestPollTimeout.ts_counter}", enable_quic=True, enable_tls=True)

        TestPollTimeout.ts_counter += 1
        self._ts = ts
        self._ts.Disk.records_config.update(
            {
                'proxy.config.diags.debug.enabled': 1,
                'proxy.config.diags.debug.tags': 'net|v_quic|quic|socket|inactivity_cop|v_iocore_net_poll',
            })

        if self.udp_poll_timeout_in is not None:
            self._ts.Disk.records_config.update({'proxy.config.udp.poll_timeout': self.udp_poll_timeout_in})

    def run(self):
        """Run the test."""
        tr = Test.AddTestRun(self.name)
        self._configure_traffic_server(tr)

        tr.Processes.Default.Command = "echo 'testing records.proxy.config.udp.poll_timeout'"
        tr.Processes.Default.StartBefore(self._ts)

        self._ts.Disk.traffic_out.Content += Testers.IncludesExpression(
            f"ET_UDP.*timeout: {self.expected_udp_poll_timeout},", "Verify UDP poll timeout.")


# Tests start.

test0 = TestPollTimeout("Test records.proxy.config.udp.poll_timeout with default value.")
test0.run()

test1 = TestPollTimeout("Test records.proxy.config.udp.poll_timeout with value of 10.", udp_poll_timeout_in=10)
test1.run()



'''
Verify correct behavior of regex_map in remap.config.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test minimum rules on load - succeed on empty file.
'''

ts = Test.MakeATSProcess("ts")
ts.Disk.remap_config.AddLine(f"")  # empty file
ts.Disk.records_config.update({f'proxy.config.url_remap.min_rules_required': 0})

tr = Test.AddTestRun("startup")
p = tr.Processes.Default
p.Command = "sh -c echo"
p.ReturnCode = 0
p.StartBefore(Test.Processes.ts)
tr.StillRunningAfter = ts



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

# Someday should add client cert to origin to exercise the
# engine interface on the other side

Test.Summary = '''
Test tls via the async interface with the sample async_engine
'''

Test.SkipUnless(
    Condition.HasOpenSSLVersion('1.1.1'),
    Condition.IsOpenSSL(),
)

# Define default ATS
ts = Test.MakeATSProcess("ts", enable_tls=True)
server = Test.MakeOriginServer("server")

ts.Setup.Copy(os.path.join(Test.Variables.AtsTestPluginsDir, 'async_engine.so'), Test.RunDirectory)

# Add info the origin server responses
server.addResponse(
    "sessionlog.json", {
        "headers": "GET / HTTP/1.1\r\nuuid: basic\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    }, {
        "headers":
            "HTTP/1.1 200 OK\r\nServer: microserver\r\nConnection: close\r\nCache-Control: max-age=3600\r\nContent-Length: 2\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": "ok"
    })

# add ssl materials like key, certificates for the server
ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")

ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.Port))

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')
ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.ssl.engine.conf_file': '{0}/ts/config/load_engine.cnf'.format(Test.RunDirectory),
        'proxy.config.ssl.async.handshake.enabled': 1,
        'proxy.config.diags.debug.enabled': 0,
        'proxy.config.diags.debug.tags': 'ssl|http'
    })

ts.Disk.MakeConfigFile('load_engine.cnf').AddLines(
    [
        'openssl_conf = openssl_init',
        '',
        '[openssl_init]',
        '',
        'engines = engine_section',
        '',
        '[engine_section]',
        '',
        'async = async_section',
        '',
        '[async_section]',
        '',
        'dynamic_path = {0}/async_engine.so'.format(Test.RunDirectory),
        '',
        'engine_id = async-test',
        '',
        'default_algorithms = RSA',
        '',
        'init = 1',
    ])

# Make a basic request.  Hopefully it goes through
tr = Test.AddTestRun("Run-Test")
tr.Processes.Default.Command = "curl -k -v -H uuid:basic -H host:example.com  https://127.0.0.1:{0}/".format(ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(Test.Processes.ts, ready=When.PortOpen(ts.Variables.ssl_port))
tr.Processes.Default.Streams.All = Testers.ContainsExpression(r"HTTP/(2|1\.1) 200", "Request succeeds")
tr.StillRunningAfter = server

ts.Disk.traffic_out.Content += Testers.ContainsExpression("Send signal to ", "The Async engine triggers")



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Test log filter.
'''

ts = Test.MakeATSProcess("ts", enable_cache=False)
replay_file = "log-filter.replays.yaml"
server = Test.MakeVerifierServerProcess("server", replay_file)
nameserver = Test.MakeDNServer("dns", default='127.0.0.1')

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'log',
        'proxy.config.net.connections_throttle': 100,
        'proxy.config.dns.nameservers': f"127.0.0.1:{nameserver.Variables.Port}",
        'proxy.config.dns.resolv_conf': 'NULL'
    })
# setup some config file for this server
ts.Disk.remap_config.AddLine('map / http://localhost:{}/'.format(server.Variables.http_port))

ts.Disk.logging_yaml.AddLines(
    '''
logging:
  filters:
    - name: only_localhost
      action: accept
      condition: chi MATCH 127.0.0.1

    - name: not_localhost
      action: accept
      condition: chi MATCH 3.3.3.3

    - name: queryparamescaper_cquuc
      action: WIPE_FIELD_VALUE
      condition: cquuc CASE_INSENSITIVE_CONTAIN password,secret,access_token,session_redirect,cardNumber,code,query,search-query,prefix,keywords,email,handle

  formats:
    - name: custom
      format: '%<cquuc>'

  logs:
    - filename: filter-test
      format: custom
      filters:
      - queryparamescaper_cquuc
      - only_localhost

    - filename: should-not-be-written
      format: custom
      filters:
      - queryparamescaper_cquuc
      - not_localhost
'''.split("\n"))

# #########################################################################
# at the end of the different test run a custom log file should exist
# Because of this we expect the testruns to pass the real test is if the
# customlog file exists and passes the format check
Test.Disk.File(os.path.join(ts.Variables.LOGDIR, 'filter-test.log'), exists=True, content='gold/filter-test.gold')

tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(nameserver)
tr.Processes.Default.StartBefore(ts)
tr.AddVerifierClientProcess("client-1", replay_file, http_ports=[ts.Variables.port])

# Wait for log file to appear, then wait one extra second to make sure TS is done writing it.
test_run = Test.AddTestRun()
test_run.Processes.Default.Command = (
    os.path.join(Test.Variables.AtsTestToolsDir, 'condwait') + ' 60 1 -f ' + os.path.join(ts.Variables.LOGDIR, 'filter-test.log'))
test_run.Processes.Default.ReturnCode = 0

# We already waited for the above, so we don't have to wait for this one.
test_run = Test.AddTestRun()
test_run.Processes.Default.Command = (
    os.path.join(Test.Variables.AtsTestToolsDir, 'condwait') + ' 1 1 -f ' +
    os.path.join(ts.Variables.LOGDIR, 'should-not-be-written.log'))
test_run.Processes.Default.ReturnCode = 1



# @file
#
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
#  Copyright 2020, Verizon Media
#

import os.path
import tempfile

Test.Summary = '''
Test traffic ramping.
'''

Test.SkipUnless(Condition.PluginExists("txn_box.so"))

RepeatCount = 100
replay_file = 'ramp.replay.yaml'

# Make a server representing a staging host to which we can ramp.
server_ramp = Test.MakeVerifierServerProcess('pv-server-staging', replay_file)
# Make it so that server-ramp is used in the txn_box config.
orig_replay_file = os.path.join(Test.TestDirectory, replay_file)
old_replay_content = open(orig_replay_file).read()
new_replay_content = old_replay_content.replace('{server_port}', str(server_ramp.Variables.http_port))
# There's no need to delete temp_replay because it will be cleaned up with the TestDirectory.
temp_replay = tempfile.NamedTemporaryFile(dir=Test.RunDirectory, delete=False)
# Reference this new replay file for all other TestRun processes.
replay_file = temp_replay.name
open(replay_file, 'w').write(new_replay_content)

tr = Test.TxnBoxTestAndRun(
    "Ramping",
    replay_file,
    remap=[('http://one.ex', 'http://three.ex', ('--key=meta.txn_box.remap', replay_file))],
    verifier_client_args=f"--verbose diag --repeat {RepeatCount}",
    verifier_server_args=f"--verbose diag")

ts = tr.Variables.TS
ts.StartBefore(server_ramp)
ts.Setup.Copy(replay_file, ts.Variables.CONFIGDIR)
ts.Setup.Copy("ramp.logging.yaml", os.path.join(ts.Variables.CONFIGDIR, "logging.yaml"))
ts.Disk.records_config.update({'proxy.config.log.max_secs_per_buffer': 1})



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Verify transaction data sink.
'''

Test.SkipUnless(Condition.PluginExists('txn_data_sink.so'),)


class TransactionDataSyncTest:

    replay_file = "transaction-with-body.replays.yaml"

    def __init__(self):
        self._setupOriginServer()
        self._setupNameserver()
        self._setupTS()

    def _setupOriginServer(self):
        self.server = Test.MakeVerifierServerProcess("server", self.replay_file)

    def _setupNameserver(self):
        self.nameserver = Test.MakeDNServer("dns", default='127.0.0.1')

    def _setupTS(self):
        self.ts = Test.MakeATSProcess("ts", enable_cache=False, enable_tls=True)
        self.ts.Disk.records_config.update(
            {
                "proxy.config.ssl.server.cert.path": f'{self.ts.Variables.SSLDir}',
                "proxy.config.ssl.server.private_key.path": f'{self.ts.Variables.SSLDir}',
                "proxy.config.ssl.client.verify.server.policy": 'PERMISSIVE',
                "proxy.config.dns.nameservers": f"127.0.0.1:{self.nameserver.Variables.Port}",
                'proxy.config.diags.debug.enabled': 1,
                'proxy.config.diags.debug.tags': 'http|txn_data_sink',
            })
        self.ts.addDefaultSSLFiles()
        self.ts.Disk.remap_config.AddLine(f'map / http://localhost:{self.server.Variables.http_port}/')
        self.ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')
        self.ts.Disk.plugin_config.AddLine('txn_data_sink.so')

        # All of the bodies that contained "not_dumped" were not configured to
        # be dumped. Therefore it is a bug if they show up in the logs.
        self.ts.Disk.traffic_out.Content += Testers.ExcludesExpression('body_not_dumped', "An unexpected body was dumped.")

        # Verify that each of the configured transaction bodies were dumped.
        self.ts.Disk.traffic_out.Content += Testers.ContainsExpression(
            'http1.1_cl_response_body_dumped', "The expected HTTP/1.1 Content-Length response body was dumped.")
        self.ts.Disk.traffic_out.Content += Testers.ContainsExpression(
            'http1.1_chunked_response_body_dumped', "The expected HTTP/1.1 chunked response body was dumped.")
        self.ts.Disk.traffic_out.Content += Testers.ContainsExpression(
            'http1.1_cl_request_body_dumped', "The expected HTTP/1.1 Content-Length request body was dumped.")
        self.ts.Disk.traffic_out.Content += Testers.ContainsExpression(
            'http1.1_chunked_request_body_dumped', "The expected HTTP/1.1 chunked request body was dumped.")
        self.ts.Disk.traffic_out.Content += Testers.ContainsExpression(
            '"http2_response_body_dumped"', "The expected HTTP/2 response body was dumped.")
        self.ts.Disk.traffic_out.Content += Testers.ContainsExpression(
            'http2_request_body_dumped', "The expected HTTP/2 request body was dumped.")

    def run(self):
        """Configure a TestRun for the test."""
        tr = Test.AddTestRun()
        tr.Processes.Default.StartBefore(self.server)
        tr.Processes.Default.StartBefore(self.nameserver)
        tr.Processes.Default.StartBefore(self.ts)
        tr.AddVerifierClientProcess(
            "client",
            self.replay_file,
            http_ports=[self.ts.Variables.port],
            https_ports=[self.ts.Variables.ssl_port],
            other_args='--thread-limit 1')


TransactionDataSyncTest().run()



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import time

Test.Summary = '''
cache_range_requests X-CRR-IMS plugin test
'''

# Test description:
# Preload the cache with the entire asset to be range requested.
# Reload remap rule with cache_range_requests plugin
# Request content through the cache_range_requests plugin

Test.SkipUnless(
    Condition.PluginExists('cache_range_requests.so'),
    Condition.PluginExists('xdebug.so'),
)
Test.ContinueOnFail = False
Test.testName = "cache_range_requests_ims"

# Define and configure ATS
ts = Test.MakeATSProcess("ts")

# Define and configure origin server
server = Test.MakeOriginServer("server")

# default root
req_chk = {
    "headers": "GET / HTTP/1.1\r\n" + "Host: www.example.com\r\n" + "uuid: none\r\n" + "\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}

res_chk = {"headers": "HTTP/1.1 200 OK\r\n" + "Connection: close\r\n" + "\r\n", "timestamp": "1469733493.993", "body": ""}

server.addResponse("sessionlog.json", req_chk, res_chk)

body = "lets go surfin now"
bodylen = len(body)

req_full = {
    "headers": "GET /path HTTP/1.1\r\n" + "Host: www.example.com\r\n" + "Accept: */*\r\n" + "Range: bytes=0-\r\n" + "\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}

res_full = {
    "headers":
        "HTTP/1.1 206 Partial Content\r\n" + "Accept-Ranges: bytes\r\n" + "Cache-Control: max-age=500\r\n" +
        "Content-Range: bytes 0-{0}/{0}\r\n".format(bodylen) + "Connection: close\r\n" + 'Etag: "772102f4-56f4bc1e6d417"\r\n' +
        "\r\n",
    "timestamp": "1469733493.993",
    "body": body
}

server.addResponse("sessionlog.json", req_full, res_full)

# cache range requests plugin remap
ts.Disk.remap_config.AddLines(
    [
        f'map http://ims http://127.0.0.1:{server.Variables.Port}' + ' @plugin=cache_range_requests.so @pparam=--consider-ims',
        f'map http://imsheader http://127.0.0.1:{server.Variables.Port}' +
        ' @plugin=cache_range_requests.so @pparam=--consider-ims' + ' @pparam=--ims-header=CrrIms',
    ])

# cache debug
ts.Disk.plugin_config.AddLine('xdebug.so --enable=x-cache')

# minimal configuration
ts.Disk.records_config.update({
    'proxy.config.diags.debug.enabled': 1,
    'proxy.config.diags.debug.tags': 'cache_range_requests',
})

curl_and_args = 'curl -s -D /dev/stdout -o /dev/stderr -x localhost:{} -H "x-debug: x-cache"'.format(ts.Variables.port)

# 0 Test - Fetch whole asset into cache
tr = Test.AddTestRun("0- range cache load")
ps = tr.Processes.Default
ps.StartBefore(server, ready=When.PortOpen(server.Variables.Port))
ps.StartBefore(Test.Processes.ts)
ps.Command = curl_and_args + ' http://ims/path -r 0-'
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: miss", "expected cache miss for load")
tr.StillRunningAfter = ts

# test inner range
# 1 Test - Fetch range into cache
tr = Test.AddTestRun("0- cache hit check")
ps = tr.Processes.Default
ps.Command = curl_and_args + ' http://ims/path -r 0-'
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: hit", "expected cache hit")
tr.StillRunningAfter = ts

# set up the IMS date field (go in the future) RFC 2616
futuretime = time.time() + 100  # seconds
futurestr = time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime(futuretime))

# 2 Test - Ensure X-CRR-IMS header results in hit-stale
tr = Test.AddTestRun("0- range X-CRR-IMS check")
ps = tr.Processes.Default
ps.Command = curl_and_args + ' http://ims/path -r 0- -H "X-CRR-IMS: {}"'.format(futurestr)
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: hit-stale", "expected cache hit-stale")
tr.StillRunningAfter = ts

futuretime += 10  # seconds
futurestr = time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime(futuretime))

# 3 Test - Ensure CrrIms header results in hit-stale
tr = Test.AddTestRun("0- range CrrIms check, override header")
ps = tr.Processes.Default
ps.Command = curl_and_args + ' http://imsheader/path -r 0- -H "CrrIms: {}"'.format(futurestr)
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: hit-stale", "expected cache hit-stale")
tr.StillRunningAfter = ts



#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import subprocess
import os

Test.Summary = '''
Test conf_remp to specify different client certificates to offer to the origin.  Loading certs/keys via plugin.
'''

ts = Test.MakeATSProcess("ts")

cafile = "{0}/signer.pem".format(Test.RunDirectory)
cafile2 = "{0}/signer2.pem".format(Test.RunDirectory)
server = Test.MakeOriginServer(
    "server",
    ssl=True,
    options={
        "--clientCA": cafile,
        "--clientverify": ""
    },
    clientcert="{0}/signed-foo.pem".format(Test.RunDirectory),
    clientkey="{0}/signed-foo.key".format(Test.RunDirectory))
server2 = Test.MakeOriginServer(
    "server2",
    ssl=True,
    options={
        "--clientCA": cafile2,
        "--clientverify": ""
    },
    clientcert="{0}/signed2-bar.pem".format(Test.RunDirectory),
    clientkey="{0}/signed-bar.key".format(Test.RunDirectory))
server3 = Test.MakeOriginServer("server3")
server.Setup.Copy("ssl/signer.pem")
server.Setup.Copy("ssl/signer2.pem")
server.Setup.Copy("ssl/signed-foo.pem")
server.Setup.Copy("ssl/signed-foo.key")
server.Setup.Copy("ssl/signed2-foo.pem")
server.Setup.Copy("ssl/signed2-bar.pem")
server.Setup.Copy("ssl/signed-bar.key")
server2.Setup.Copy("ssl/signer.pem")
server2.Setup.Copy("ssl/signer2.pem")
server2.Setup.Copy("ssl/signed-foo.pem")
server2.Setup.Copy("ssl/signed-foo.key")
server2.Setup.Copy("ssl/signed2-foo.pem")
server2.Setup.Copy("ssl/signed2-bar.pem")
server2.Setup.Copy("ssl/signed-bar.key")

request_header = {"headers": "GET / HTTP/1.1\r\nHost: example.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {
    "headers": "HTTP/1.1 200 OK\r\nConnection: close\r\nCache-Control: no-cache\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
server.addResponse("sessionlog.json", request_header, response_header)
request_header = {"headers": "GET / HTTP/1.1\r\nHost: bar.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {
    "headers": "HTTP/1.1 200 OK\r\nCache-Control: no-cache\r\nConnection: close\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
server.addResponse("sessionlog.json", request_header, response_header)

ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")
ts.addSSLfile("ssl/signed-foo.pem")
ts.addSSLfile("ssl/signed-foo.key")
ts.addSSLfile("ssl/signed2-foo.pem")
ts.addSSLfile("ssl/signed-bar.pem")
ts.addSSLfile("ssl/signed2-bar.pem")
ts.addSSLfile("ssl/signed-bar.key")

ts.Disk.sni_yaml.AddLine('sni:')
ts.Disk.sni_yaml.AddLine('- fqdn: random')
ts.Disk.sni_yaml.AddLine('  verify_server_properties: NONE')
snipath = ts.Disk.sni_yaml.AbsPath

Test.PrepareTestPlugin(os.path.join(Test.Variables.AtsTestPluginsDir, 'ssl_secret_load_test.so'), ts)

shortdir = ts.Variables.SSLDir[0:ts.Variables.SSLDir.rfind("/")]

ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.client.verify.server.policy': 'PERMISSIVE',
        'proxy.config.ssl.client.cert.path': '{0}'.format(shortdir),
        'proxy.config.ssl.client.cert.filename': 'signed-foo.pem',
        'proxy.config.ssl.client.private_key.path': '{0}'.format(shortdir),
        'proxy.config.ssl.client.private_key.filename': 'signed-foo.key',
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.url_remap.pristine_host_hdr': 1,
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'ssl_secret_load|http|ssl',
    })

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

ts.Disk.remap_config.AddLine(
    'map /case1 https://127.0.0.1:{0}/ @plugin=conf_remap.so @pparam=proxy.config.ssl.client.cert.filename={1} plugin=conf_remap.so @pparam=proxy.config.ssl.client.private_key.filename={2}'
    .format(server.Variables.SSL_Port, "signed-foo.pem", "signed-foo.key"))
ts.Disk.remap_config.AddLine(
    'map /badcase1 https://127.0.0.1:{0}/ @plugin=conf_remap.so @pparam=proxy.config.ssl.client.cert.filename={1} plugin=conf_remap.so @pparam=proxy.config.ssl.client.private_key.filename={2}'
    .format(server.Variables.SSL_Port, "signed2-foo.pem", "signed-foo.key"))
ts.Disk.remap_config.AddLine(
    'map /case2 https://127.0.0.1:{0}/ @plugin=conf_remap.so @pparam=proxy.config.ssl.client.cert.filename={1} plugin=conf_remap.so @pparam=proxy.config.ssl.client.private_key.filename={2}'
    .format(server2.Variables.SSL_Port, "signed2-foo.pem", "signed-foo.key"))
ts.Disk.remap_config.AddLine(
    'map /badcase2 https://127.0.0.1:{0}/ @plugin=conf_remap.so @pparam=proxy.config.ssl.client.cert.filename={1} plugin=conf_remap.so @pparam=proxy.config.ssl.client.private_key.filename={2}'
    .format(server2.Variables.SSL_Port, "signed-foo.pem", "signed-foo.key"))

# Should succeed
tr = Test.AddTestRun("Connect with correct client cert to first server")
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(server2)
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.StillRunningAfter = server2
tr.Processes.Default.Command = "curl -H host:example.com  http://127.0.0.1:{0}/case1".format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Check response")

# Should fail
trfail = Test.AddTestRun("Connect with bad client cert to first server")
trfail.StillRunningAfter = ts
trfail.StillRunningAfter = server
trfail.StillRunningAfter = server2
trfail.Processes.Default.Command = 'curl -H host:example.com  http://127.0.0.1:{0}/badcase1'.format(ts.Variables.port)
trfail.Processes.Default.ReturnCode = 0
trfail.Processes.Default.Streams.stdout = Testers.ContainsExpression("Could Not Connect", "Check response")

# Should succeed
trbar = Test.AddTestRun("Connect with correct client cert to second server")
trbar.StillRunningAfter = ts
trbar.StillRunningAfter = server
trbar.StillRunningAfter = server2
trbar.Processes.Default.Command = "curl -H host:bar.com  http://127.0.0.1:{0}/case2".format(ts.Variables.port)
trbar.Processes.Default.ReturnCode = 0
trbar.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Check response")

# Should fail
trbarfail = Test.AddTestRun("Connect with bad client cert to second server")
trbarfail.StillRunningAfter = ts
trbarfail.StillRunningAfter = server
trbarfail.StillRunningAfter = server2
trbarfail.Processes.Default.Command = 'curl -H host:bar.com  http://127.0.0.1:{0}/badcase2'.format(ts.Variables.port)
trbarfail.Processes.Default.ReturnCode = 0
trbarfail.Processes.Default.Streams.stdout = Testers.ContainsExpression("Could Not Connect", "Check response")

# Test the case of updating certificate contents without changing file name.
trupdate = Test.AddTestRun("Update client cert file in place")
trupdate.StillRunningAfter = ts
trupdate.StillRunningAfter = server
trupdate.StillRunningAfter = server2
# in the config/ssl directory for records.yaml
trupdate.Setup.CopyAs("ssl/signed-foo.pem", ".", "{0}/signed2-foo.pem".format(ts.Variables.SSLDir))
trupdate.Processes.Default.Command = 'traffic_ctl config set proxy.config.ssl.client.cert.path {0}/; touch {1}'.format(
    shortdir, snipath)
# Need to copy over the environment so traffic_ctl knows where to find the unix domain socket
trupdate.Processes.Default.Env = ts.Env
trupdate.Processes.Default.ReturnCode = 0

tr2reload = Test.AddTestRun("Reload config")
tr2reload.StillRunningAfter = ts
tr2reload.StillRunningAfter = server
tr2reload.StillRunningAfter = server2
tr2reload.Processes.Default.Command = 'traffic_ctl config reload'
# Need to copy over the environment so traffic_ctl knows where to find the unix domain socket
tr2reload.Processes.Default.Env = ts.Env
tr2reload.Processes.Default.ReturnCode = 0

tr3bar = Test.AddTestRun("Make request with other foo.  badcase1 should now work")
# Wait for the reload to complete
tr3bar.Processes.Default.StartBefore(server3, ready=When.FileContains(ts.Disk.diags_log.Name, 'sni.yaml finished loading', 3))
tr3bar.StillRunningAfter = ts
tr3bar.StillRunningAfter = server
tr3bar.StillRunningAfter = server2
tr3bar.Processes.Default.Command = 'curl  -H host:foo.com http://127.0.0.1:{0}/badcase1'.format(ts.Variables.port)
tr3bar.Processes.Default.ReturnCode = 0
tr3bar.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Check response")

# Test the hot-reload feature.  Update cert file and wait about.  Should update without reload
trupdate = Test.AddTestRun("Update signed-foo cert file in place")
trupdate.StillRunningAfter = ts
trupdate.StillRunningAfter = server
trupdate.Setup.CopyAs("ssl/signed2-foo.pem", ".", "{0}/signed-foo.pem".format(ts.Variables.SSLDir))
# For some reason the Setup.CopyAs does not change the modification time, so we touch
trupdate.Processes.Default.Command = 'touch {0}/signed-foo.pem {0}/signed-foo.key'.format(ts.Variables.SSLDir)
# Need to copy over the environment so traffic_ctl knows where to find the unix domain socket
trupdate.Processes.Default.Env = ts.Env
trupdate.Processes.Default.ReturnCode = 0

# The plugin will pull every 3 seconds.  So wait 4 seconds and test again.
# case1 should fail
# badcase1 should succeed
tr = Test.AddTestRun("Retest case1")
tr.DelayStart = 4
tr.ReturnCode = 0
tr.StillRunningAfter = server
tr.StillRunningAfter = server2
tr.StillRunningAfter = ts
tr.Processes.Default.Command = "curl -H host:example.com  http://127.0.0.1:{0}/case1".format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = Testers.ContainsExpression("Could Not Connect", "Check response")

tr = Test.AddTestRun("Retest badcase1")
tr.ReturnCode = 0
tr.StillRunningAfter = server
tr.StillRunningAfter = server2
tr.StillRunningAfter = ts
tr.Processes.Default.Command = "curl -H host:example.com  http://127.0.0.1:{0}/badcase1".format(ts.Variables.port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = Testers.ExcludesExpression("Could Not Connect", "Check response")



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = "Test start up of Traffic server with configuration modification of starting port"

ts = Test.MakeATSProcess("ts", select_ports=False)
ts.Variables.port = 8090
ts.Disk.records_config.update({
    'proxy.config.http.server_ports': str(ts.Variables.port),
})
ts.Ready = When.PortOpen(ts.Variables.port)
t = Test.AddTestRun("Test traffic server started properly")
t.Processes.Default.StartBefore(ts)
t.Command = "curl 127.0.0.1:{port}".format(port=ts.Variables.port)
t.ReturnCode = 0
t.StillRunningAfter = ts



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import uuid

Test.Summary = '''
Test that incrementing the cache generation acts like a cache clear
'''
Test.ContinueOnFail = True
# Define default ATS
ts = Test.MakeATSProcess("ts")

# setup some config file for this server
ts.Disk.records_config.update(
    {
        'proxy.config.body_factory.enable_customizations': 3,  # enable domain specific body factory
        'proxy.config.http.cache.generation': -1,  # Start with cache turned off
        'proxy.config.config_update_interval_ms': 1,
    })

ts.Disk.plugin_config.AddLine('xdebug.so --enable=x-cache,x-cache-key,via,x-cache-generation')
ts.Disk.remap_config.AddLines(
    [
        'map /default/ http://127.0.0.1/ @plugin=generator.so',
        # line 2
        'map /generation1/ http://127.0.0.1/' + ' @plugin=conf_remap.so @pparam=proxy.config.http.cache.generation=1' +
        ' @plugin=generator.so',
        # line 3
        'map /generation2/ http://127.0.0.1/' + ' @plugin=conf_remap.so @pparam=proxy.config.http.cache.generation=2' +
        ' @plugin=generator.so',
    ])

objectid = uuid.uuid4()
# first test is a miss for default
tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.0.0.1:{0}/default/cache/10/{1}" -H "x-debug: x-cache,x-cache-key,via,x-cache-generation" --verbose'.format(
    ts.Variables.port, objectid)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.Streams.All = "gold/miss_default-1.gold"

# Second touch is a HIT for default.
tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.0.0.1:{0}/default/cache/10/{1}" -H "x-debug: x-cache,x-cache-key,via,x-cache-generation" --verbose'.format(
    ts.Variables.port, objectid)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = "gold/hit_default-1.gold"

# Call traffic_ctrl to set new generation
tr = Test.AddTestRun()
tr.Processes.Default.Command = f'traffic_ctl --debug config set proxy.config.http.cache.generation 77'
tr.Processes.Default.ForceUseShell = False
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Env = ts.Env  # set the environment for traffic_control to run in

# new generation should first be a miss.
tr = Test.AddTestRun()
tr.DelayStart = 15  # delay start of test run to allow previous command to take effect
# create a new traffic_ctrl call and the environment
tr.Processes.Default.Command = 'curl "http://127.0.0.1:{0}/default/cache/10/{1}" -H "x-debug: x-cache,x-cache-key,via,x-cache-generation" --verbose'.format(
    ts.Variables.port, objectid)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = "gold/miss_default77.gold"

# new generation should should now hit.
tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.0.0.1:{0}/default/cache/10/{1}" -H "x-debug: x-cache,x-cache-key,via,x-cache-generation" --verbose'.format(
    ts.Variables.port, objectid)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = "gold/hit_default77.gold"

# should still hit.
tr = Test.AddTestRun()
tr.Processes.Default.Command = 'curl "http://127.0.0.1:{0}/default/cache/10/{1}" -H "x-debug: x-cache,x-cache-key,via,x-cache-generation" --verbose'.format(
    ts.Variables.port, objectid)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = "gold/hit_default77.gold"



'''
Test cached responses and requests with bodies
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test revalidating cached objects
'''
testName = "RevalidateCacheObject"
Test.ContinueOnFail = True


class CachedHeaderValidationTest:
    replay_file = "replays/cache-test.replay.yaml"

    def __init__(self):
        self.setupOriginServer()
        self.setupTS()

    def setupOriginServer(self):
        self.server = Test.MakeVerifierServerProcess("cached-header-verifier-server", self.replay_file)

    def setupTS(self):
        self.ts = Test.MakeATSProcess("ts", enable_tls=True)
        self.ts.Disk.plugin_config.AddLine('xdebug.so --enable=x-cache,x-cache-key,via')
        self.ts.Disk.records_config.update(
            {
                'proxy.config.diags.debug.enabled': 1,
                'proxy.config.diags.debug.tags': 'http',
                'proxy.config.http.response_via_str': 3,
            })
        self.ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(self.server.Variables.http_port))

    def runTraffic(self):
        tr = Test.AddTestRun()
        tr.AddVerifierClientProcess("cached-header-verifier-client", self.replay_file, http_ports=[self.ts.Variables.port])
        tr.Processes.Default.StartBefore(self.server)
        tr.Processes.Default.StartBefore(self.ts)
        tr.StillRunningAfter = self.ts
        tr.StillRunningAfter = self.server

    def run(self):
        self.runTraffic()


CachedHeaderValidationTest().run()



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from enum import Enum

Test.Summary = 'Exercise stats-over-http plugin'
Test.SkipUnless(Condition.PluginExists('stats_over_http.so'))
Test.ContinueOnFail = True


class StatsOverHttpPluginTest:
    """
    https://docs.trafficserver.apache.org/en/latest/admin-guide/plugins/stats_over_http.en.html
    """

    class State(Enum):
        """
        State of process
        """
        INIT = 0
        RUNNING = 1

    def __init__(self):
        self.state = self.State.INIT
        self.__setupTS()

    def __setupTS(self):
        self.ts = Test.MakeATSProcess("ts")

        self.ts.Disk.plugin_config.AddLine('stats_over_http.so _stats')

        self.ts.Disk.records_config.update(
            {
                "proxy.config.http.server_ports": f"{self.ts.Variables.port}",
                "proxy.config.diags.debug.enabled": 1,
                "proxy.config.diags.debug.tags": "stats_over_http"
            })

    def __checkProcessBefore(self, tr):
        if self.state == self.State.RUNNING:
            tr.StillRunningBefore = self.ts
        else:
            tr.Processes.Default.StartBefore(self.ts)
            self.state = self.State.RUNNING

    def __checkProcessAfter(self, tr):
        assert (self.state == self.State.RUNNING)
        tr.StillRunningAfter = self.ts

    def __testCase0(self):
        tr = Test.AddTestRun()
        self.__checkProcessBefore(tr)
        tr.Processes.Default.Command = f"curl -vs --http1.1 http://127.0.0.1:{self.ts.Variables.port}/_stats"
        tr.Processes.Default.ReturnCode = 0
        tr.Processes.Default.Streams.stdout = "gold/stats_over_http_0_stdout.gold"
        tr.Processes.Default.Streams.stderr = "gold/stats_over_http_0_stderr.gold"
        tr.Processes.Default.TimeOut = 3
        self.__checkProcessAfter(tr)

    def run(self):
        self.__testCase0()


StatsOverHttpPluginTest().run()



'''
Verify correct behavior for default_inactivity_timeout.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = __doc__


class TestDefaultInactivityTimeout:
    """Configure a test for default_inactivity_test."""

    replay_file = "replay/default_inactivity_timeout.replay.yaml"
    client_gold_file = 'gold/client_default_inactivity_timeout.gold'
    client_counter: int = 0
    ts_counter: int = 0
    server_counter: int = 0

    def __init__(self, name: str, use_override: bool):
        """Initialize the test.

        :param name: The name of the test.
        :param use_override: Whether to use the override rather than the global
        configuration.
        """
        self.name = name
        self.use_override = use_override

    def _configure_server(self, tr: 'TestRun') -> None:
        """Configure the server.

        :param tr: The TestRun object to associate the server process with.
        """
        server = tr.AddVerifierServerProcess(f"server_{TestDefaultInactivityTimeout.server_counter}", self.replay_file)
        TestDefaultInactivityTimeout.server_counter += 1
        self._server = server

    def _configure_traffic_server(self, tr: 'TestRun') -> None:
        """Configure Traffic Server.

        :param tr: The TestRun object to associate the ts process with.
        :return: A Traffic Server Test process.
        """
        ts = tr.MakeATSProcess(f"ts-{TestDefaultInactivityTimeout.ts_counter}")
        TestDefaultInactivityTimeout.ts_counter += 1
        self._ts = ts

        debug_tags = 'http|cache|socket|net_queue|inactivity_cop|conf_remap'
        ts.Disk.records_config.update({
            'proxy.config.diags.debug.enabled': 1,
            'proxy.config.diags.debug.tags': debug_tags,
        })

        origin_port = self._server.Variables.http_port
        remap_line = f'map / http://127.0.0.1:{origin_port}'

        if self.use_override:
            remap_line += (' @plugin=conf_remap.so '
                           '@pparam=proxy.config.net.default_inactivity_timeout=2')
        else:
            ts.Disk.records_config.update({
                'proxy.config.net.default_inactivity_timeout': 2,
            })

        ts.Disk.remap_config.AddLine(remap_line)
        ts.Disk.traffic_out.Content += Testers.ContainsExpression(
            'timed out due to default inactivity timeout', 'Verify that the default inactivity timeout was triggered.')

    def run(self) -> None:
        """Run the test."""
        tr = Test.AddTestRun(self.name)
        self._configure_server(tr)
        self._configure_traffic_server(tr)

        tr.Processes.Default.StartBefore(self._server)
        tr.Processes.Default.StartBefore(self._ts)
        tr.AddVerifierClientProcess(
            f'client-{TestDefaultInactivityTimeout.client_counter}', self.replay_file, http_ports=[self._ts.Variables.port])
        TestDefaultInactivityTimeout.client_counter += 1

        # Set up expectectations for the timeout closing the connection.
        tr.Processes.Default.ReturnCode = 1
        tr.Processes.Default.Streams.all = self.client_gold_file


test = TestDefaultInactivityTimeout("global config", use_override=False)
test.run()

test = TestDefaultInactivityTimeout("override config", use_override=True)
test.run()



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import urllib.parse

Test.Summary = '''
Test prefetch.so plugin (simple mode).
'''

origin = Test.MakeOriginServer("origin")

asset_name = 'request.txt'
pf_name = 'prefetch.txt'
pf_header = f'Cmcd-Request: foo=12,nor="{pf_name}",bar=42'

request_header = {
    "headers":
        f"GET /tests/{asset_name} HTTP/1.1\r\n"
        "Host: does.not.matter\r\n"  # But cant be omitted
        f"{pf_header}\r\n"
        "\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header = {
    "headers": "HTTP/1.1 200 OK\r\n"
               "Connection: close\r\n"
               "Cache-control: max-age=60\r\n"
               "\r\n",
    "timestamp": "1469733493.993",
    "body": f"This is the body for {asset_name}\n"
}
origin.addResponse("sessionlog.json", request_header, response_header)

# query string
query_name = 'query?this=foo&that'
query_pf_name = 'query?bar=baz'
query_pf_header = f'Cmcd-Request: nor="{query_pf_name}"'

# nor field may be percent encoded
query_pf_perc_name = urllib.parse.quote(query_pf_name)
query_pf_perc_header = f'Cmcd-Request: nor="{query_pf_perc_name}"'

request_header = {
    "headers":
        f"GET /tests/{query_name} HTTP/1.1\r\n"
        "Host: does.not.matter\r\n"  # But cant be omitted
        f"{query_pf_perc_header}\r\n"
        "\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header = {
    "headers": "HTTP/1.1 200 OK\r\n"
               "Connection: close\r\n"
               "Cache-control: max-age=60\r\n"
               "\r\n",
    "timestamp": "1469733493.993",
    "body": f"This is the body for {query_name}\n"
}
origin.addResponse("sessionlog.json", request_header, response_header)

# setup the prefetched assets
names = [pf_name, query_pf_name]

for name in names:
    request_header = {
        "headers":
            f"GET /tests/{name} HTTP/1.1\r\n"
            "Host: does.not.matter\r\n"  # But cant be omitted
            "\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    }
    response_header = {
        "headers": "HTTP/1.1 200 OK\r\n"
                   "Connection: close\r\n"
                   "Cache-control: max-age=60\r\n"
                   "\r\n",
        "timestamp": "1469733493.993",
        "body": f"This is the body for {name}\n"
    }
    origin.addResponse("sessionlog.json", request_header, response_header)

# prefetch from root
root_name = 'root.txt'
root_header = f'Cmcd-Request: nor="rooted"'

request_header = {
    "headers":
        f"GET /{root_name} HTTP/1.1\r\n"
        "Host: does.not.matter\r\n"  # But cant be omitted
        f"{root_header}\r\n"
        "\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header = {
    "headers": "HTTP/1.1 200 OK\r\n"
               "Connection: close\r\n"
               "Cache-control: max-age=60\r\n"
               "\r\n",
    "timestamp": "1469733493.993",
    "body": f"This is the body for {root_name}\n"
}
origin.addResponse("sessionlog.json", request_header, response_header)

request_header = {
    "headers":
        f"GET /rooted HTTP/1.1\r\n"
        "Host: does.not.matter\r\n"  # But cant be omitted
        "\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header = {
    "headers": "HTTP/1.1 200 OK\r\n"
               "Connection: close\r\n"
               "Cache-control: max-age=60\r\n"
               "\r\n",
    "timestamp": "1469733493.993",
    "body": f"This is the body for rooted\n"
}
origin.addResponse("sessionlog.json", request_header, response_header)

# ignore if cmcd-request nrr= found
crr_name = 'crr.txt'
crr_header = f'Cmcd-Request: foo=12,nor="{crr_name}",bar=42,nrr="0-"'
request_header = {
    "headers":
        f"GET /tests/{crr_name} HTTP/1.1\r\n"
        "Host: does.not.matter\r\n"  # But cant be omitted
        f"{crr_header}\r\n"
        "\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header = {
    "headers": "HTTP/1.1 200 OK\r\n"
               "Connection: close\r\n"
               "Cache-control: max-age=60\r\n"
               "\r\n",
    "timestamp": "1469733493.993",
    "body": f"This is the body for {crr_name}\n"
}
origin.addResponse("sessionlog.json", request_header, response_header)

# allows for multiple ats on localhost
dns = Test.MakeDNServer("dns")

# next hop trafficserver instance
ts1 = Test.MakeATSProcess("ts1")
ts1.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'prefetch|http',
        'proxy.config.dns.nameservers': f"127.0.0.1:{dns.Variables.Port}",
        'proxy.config.dns.resolv_conf': "NULL",
        'proxy.config.http.parent_proxy.self_detect': 0,
    })
dns.addRecords(records={f"ts1": ["127.0.0.1"]})
ts1.Disk.remap_config.AddLine(
    f"map / http://127.0.0.1:{origin.Variables.Port}" + " @plugin=cachekey.so @pparam==--sort-params=true"
    " @plugin=prefetch.so @pparam==--front=false")

ts1.Disk.logging_yaml.AddLines(
    '''
logging:
 formats:
  - name: custom
    format: '%<cquup> %<pssc> %<crc> %<cwr> %<pscl> %<{X-CDN-Prefetch}cqh>'
 logs:
  - filename: transaction
    format: custom
'''.split("\n"))

ts0 = Test.MakeATSProcess("ts0")
ts0.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'prefetch|http',
        'proxy.config.dns.nameservers': f"127.0.0.1:{dns.Variables.Port}",
        'proxy.config.dns.resolv_conf': "NULL",
        'proxy.config.http.parent_proxy.self_detect': 0,
    })

dns.addRecords(records={f"ts0": ["127.0.0.1"]})
ts0.Disk.remap_config.AddLine(
    f"map http://ts0 http://ts1:{ts1.Variables.port}" + " @plugin=cachekey.so @pparam=--sort-params=true"
    " @plugin=prefetch.so" + " @pparam=--front=true" + " @pparam=--fetch-policy=simple" + " @pparam=--cmcd-nor=true")

ts0.Disk.logging_yaml.AddLines(
    '''
logging:
 formats:
  - name: custom
    format: '%<cquup> %<pssc> %<crc> %<cwr> %<pscl> %<{X-CDN-Prefetch}cqh>'
 logs:
  - filename: transaction
    format: custom
'''.split("\n"))

# start everything up
tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(origin)
tr.Processes.Default.StartBefore(dns)
tr.Processes.Default.StartBefore(ts0)
tr.Processes.Default.StartBefore(ts1)
tr.Processes.Default.Command = 'echo start TS, TSH_N, HTTP origin and DNS.'
tr.Processes.Default.ReturnCode = 0

# attempt to get normal asset
tr = Test.AddTestRun()
tr.Processes.Default.Command = (f"curl --verbose --proxy 127.0.0.1:{ts0.Variables.port} http://ts0/tests/{asset_name}")
tr.Processes.Default.ReturnCode = 0

# issue curl form same asset, with prefetch
tr = Test.AddTestRun()
tr.DelayStart = 1
tr.Processes.Default.Command = (
    f"curl --verbose --proxy 127.0.0.1:{ts0.Variables.port} http://ts0/tests/{asset_name} -H \'{pf_header}\'")
tr.Processes.Default.ReturnCode = 0

# fetch the prefetched asset (only cached on ts1)
tr = Test.AddTestRun()
tr.DelayStart = 1
tr.Processes.Default.Command = (f"curl --verbose --proxy 127.0.0.1:{ts0.Variables.port} http://ts0/tests/{pf_name}")
tr.Processes.Default.ReturnCode = 0

# attempt to prefetch again
tr = Test.AddTestRun()
tr.Processes.Default.Command = (
    f"curl --verbose --proxy 127.0.0.1:{ts0.Variables.port} http://ts0/tests/{asset_name} -H \'{pf_header}\'")
tr.Processes.Default.ReturnCode = 0

# request the prefetched asset
tr = Test.AddTestRun()
tr.Processes.Default.Command = (f"curl --verbose --proxy 127.0.0.1:{ts0.Variables.port} http://ts0/tests/{pf_name}")
tr.Processes.Default.ReturnCode = 0

# prefetch using query params with query prefetch perc encoded
tr = Test.AddTestRun()
tr.Processes.Default.Command = (
    f"curl --verbose --proxy 127.0.0.1:{ts0.Variables.port} \'http://ts0/tests/{query_name}\' -H \'{query_pf_perc_header}\'")
tr.Processes.Default.ReturnCode = 0

# request the prefetched asset without perc encoding
tr = Test.AddTestRun()
tr.Processes.Default.Command = (f"curl --verbose --proxy 127.0.0.1:{ts0.Variables.port} \'http://ts0/tests/{query_pf_name}\'")
tr.Processes.Default.ReturnCode = 0

# ensure root path prefetch works
tr = Test.AddTestRun()
tr.Processes.Default.Command = (
    f"curl --verbose --proxy 127.0.0.1:{ts0.Variables.port} \'http://ts0/{root_name}\' -H \'{root_header}\'")
tr.Processes.Default.ReturnCode = 0

# ensure request with nrr= field is skipped
tr = Test.AddTestRun()
tr.Processes.Default.Command = (
    f"curl --verbose --proxy 127.0.0.1:{ts0.Variables.port} \'http://ts0/{crr_name}\' -H \'{crr_header}\'")
tr.Processes.Default.ReturnCode = 0

condwaitpath = os.path.join(Test.Variables.AtsTestToolsDir, 'condwait')

ts0log = os.path.join(ts0.Variables.LOGDIR, 'transaction.log')
Test.AddAwaitFileContainsTestRun('Await ts transactions to finish logging.', ts0log, 'crr.txt')

ts1log = os.path.join(ts1.Variables.LOGDIR, 'transaction.log')
Test.AddAwaitFileContainsTestRun('Await ts1 transactions to finish logging.', ts1log, 'crr.txt')

tr = Test.AddTestRun()
tr.Processes.Default.Command = (f"cat {ts0log}")
tr.Streams.stdout = "prefetch_cmcd0.gold"
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = (f"cat {ts1log}")
tr.Streams.stdout = "prefetch_cmcd1.gold"
tr.Processes.Default.ReturnCode = 0



'''
JSONRPC Schema test. This test will run a basic request/response schema validation.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import sys
import tempfile
from string import Template

Test.Summary = 'Test jsonrpc admin API'

# set the schema folder.
schema_folder = os.path.join(Test.TestDirectory, '..', '..', '..', "src", "mgmt", "rpc", "schema")


def substitute_context_in_file(process, file, context):
    '''
    Perform substitution based on the passed context dict. This function will return a new path for the substituted file.
    '''
    if os.path.isdir(file):
        raise ValueError(f"Mapping substitution not supported for directories.")

    with open(os.path.join(process.TestDirectory, file), 'r') as req_file:
        req_template = Template(req_file.read())
        req_content = req_template.substitute(context)
        tf = tempfile.NamedTemporaryFile(delete=False, dir=process.RunDirectory, suffix=f"_{os.path.basename(file)}")
        file = tf.name
        with open(file, "w") as new_req_file:
            new_req_file.write(req_content)

    return file


def add_testrun_for_jsonrpc_request(
        test_description, request_file_name, params_schema_file_name=None, result_schema_file_name=None, context=None):
    '''
    Simple wrapper around the AddJsonRPCClientRequest method.

    context:
      This can be used if a template substitution is needed in the request file.

    stdout_testers:
      Testers to be run on the output stream.

    params_schema_file_name:
        Schema file to validate the request 'params' field.

    result_schema_file_name:
        Schema file to validate the response 'result' field.
    '''
    tr = Test.AddTestRun(test_description)
    tr.Setup.Copy(request_file_name)

    if context:
        request_file_name = substitute_context_in_file(tr, request_file_name, context)

    request_schema_file_name = os.path.join(schema_folder, "jsonrpc_request_schema.json")
    tr.AddJsonRPCClientRequest(
        ts,
        file=os.path.join(ts.RunDirectory, os.path.basename(request_file_name)),
        schema_file_name=request_schema_file_name,
        params_field_schema_file_name=params_schema_file_name)

    tr.Processes.Default.ReturnCode = 0

    response_schema_file_name = os.path.join(schema_folder, "jsonrpc_response_schema.json")
    tr.Processes.Default.Streams.stdout = Testers.JSONRPCResponseSchemaValidator(
        schema_file_name=response_schema_file_name, result_field_schema_file_name=result_schema_file_name)

    tr.StillRunningAfter = ts
    return tr


ts = Test.MakeATSProcess('ts')

Test.testName = 'Basic JSONRPC API test'

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'rpc|filemanager|http|cache',
        'proxy.config.jsonrpc.filename': "jsonrpc.yaml",  # We will be using this record to tests some RPC API.
    })

# One of the API's will be checking the storage. Need this to get a response with content.
storage_path = os.path.join(Test.RunDirectory, "ts", "storage")
ts.Disk.storage_config.AddLine(f"{storage_path} 512M")

# The following tests will only validate the jsonrpc message, it will not run any validation on the content of the 'result' or 'params'
# of the jsonrpc message. This should be added once the schemas are available.

# jsonrpc 2.0 schema file. This will not check the param fields.

success_schema_file_name_name = os.path.join(schema_folder, "success_response_schema.json")
# admin_lookup_records

params_schema_file_name = os.path.join(schema_folder, "admin_lookup_records_params_schema.json")
first = add_testrun_for_jsonrpc_request(
    "Test admin_lookup_records",
    request_file_name='json/admin_lookup_records_req_1.json',
    params_schema_file_name=params_schema_file_name,
    context={'record_name': 'proxy.config.jsonrpc.filename'})
first.Processes.Default.StartBefore(ts)

add_testrun_for_jsonrpc_request(
    "Test admin_lookup_records w/error", request_file_name='json/admin_lookup_records_req_invalid_rec.json')

add_testrun_for_jsonrpc_request(
    "Test admin_lookup_records",
    request_file_name='json/admin_lookup_records_req_1.json',
    context={'record_name': 'proxy.config.jsonrpc.filename'})

add_testrun_for_jsonrpc_request(
    "Test admin_lookup_records w/error", request_file_name='json/admin_lookup_records_req_invalid_rec.json')

add_testrun_for_jsonrpc_request(
    "Test admin_lookup_records w/error",
    request_file_name='json/admin_lookup_records_req_multiple.json',
    context={'record_name': 'proxy.config.jsonrpc.filename'})

add_testrun_for_jsonrpc_request(
    "Test admin_lookup_records w/error",
    request_file_name='json/admin_lookup_records_req_metric.json',
    context={'record_name_regex': 'proxy.process.http.total_client_connections_ipv4*'})

# admin_config_set_records
add_testrun_for_jsonrpc_request(
    "Test admin_lookup_records w/error",
    request_file_name='json/admin_config_set_records_req.json',
    context={
        'record_name': 'proxy.config.jsonrpc.filename',
        'record_value': 'test_jsonrpc.yaml'
    })

# admin_config_reload
add_testrun_for_jsonrpc_request(
    "Test admin_config_reload",
    request_file_name='json/admin_config_reload_req.json',
    result_schema_file_name=success_schema_file_name_name)

# admin_host_set_status
add_testrun_for_jsonrpc_request(
    "Test admin_host_set_status",
    request_file_name='json/admin_host_set_status_req.json',
    context={
        'operation': 'up',
        'host': 'my.test.host.trafficserver.com'
    })

# admin_host_set_status
add_testrun_for_jsonrpc_request(
    "Test admin_host_set_status",
    request_file_name='json/admin_host_set_status_req.json',
    context={
        'operation': 'down',
        'host': 'my.test.host.trafficserver.com'
    })

# admin_server_start_drain
add_testrun_for_jsonrpc_request(
    "Test admin_server_start_drain",
    request_file_name='json/method_call_no_params.json',
    context={'method': 'admin_server_start_drain'})

add_testrun_for_jsonrpc_request(
    "Test admin_server_start_drain",
    request_file_name='json/method_call_no_params.json',
    context={'method': 'admin_server_start_drain'})

# admin_server_stop_drain
add_testrun_for_jsonrpc_request(
    "Test admin_server_stop_drain",
    request_file_name='json/method_call_no_params.json',
    context={'method': 'admin_server_stop_drain'})

# admin_storage_get_device_status
add_testrun_for_jsonrpc_request(
    "Test admin_storage_get_device_status",
    request_file_name='json/admin_storage_x_device_status_req.json',
    context={
        'method': 'admin_storage_get_device_status',
        'device': f'{storage_path}/cache.db'
    })

# admin_storage_set_device_offline
add_testrun_for_jsonrpc_request(
    "Test admin_storage_set_device_offline",
    request_file_name='json/admin_storage_x_device_status_req.json',
    context={
        'method': 'admin_storage_set_device_offline',
        'device': f'{storage_path}/cache.db'
    })

# admin_plugin_send_basic_msg
add_testrun_for_jsonrpc_request(
    "Test admin_plugin_send_basic_msg",
    request_file_name='json/admin_plugin_send_basic_msg_req.json',
    result_schema_file_name=success_schema_file_name_name)



"""
Test parent_retry config settings
"""
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.testName = "Test parent_retry settings"
Test.ContinueOnFail = True


class ParentRetryTest:
    """
    Test loading parent.config with parent_retry setting enabled
    """
    ts_parent_hostname = "localhost:8081"

    def __init__(self):
        """Initialize the test."""
        self._configure_ts_child()

    def _configure_ts_child(self):
        self.ts_child = Test.MakeATSProcess("ts_child")
        self.ts_child.Disk.parent_config.AddLine(
            f'dest_domain=. method=get parent="{self.ts_parent_hostname}" parent_retry=unavailable_server_retry unavailable_server_retry_responses="502,503"'
        )

    def run(self):
        tr = Test.AddTestRun()
        tr.Processes.Default.StartBefore(self.ts_child)
        tr.Processes.Default.Command = f'curl "{self.ts_child.Variables.port}" --verbose'
        tr.StillRunningAfter = self.ts_child


ParentRetryTest().run()



'''
Verify correct URI parsing behavior.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Verify correct URI parsing behavior.
'''

ts = Test.MakeATSProcess("ts")
replay_file = "uri.replay.yaml"
server = Test.MakeVerifierServerProcess("server", replay_file)
ts.Disk.records_config.update({
    'proxy.config.diags.debug.enabled': 1,
    'proxy.config.diags.debug.tags': 'http|cache|url',
})
ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.http_port))
tr = Test.AddTestRun("Verify correct URI parsing behavior.")
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(ts)
tr.AddVerifierClientProcess("client", replay_file, http_ports=[ts.Variables.port], other_args='--thread-limit 1')



#!/usr/bin/env python3
#
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
import fileinput
# import pprint


def bases():
    pass


def indentp(str, indent):
    print("{}{}".format(" " * indent * 2, str))


def print_header():
    print(
        """
/*
  Licensed to the Apache Software Foundation (ASF) under one
  or more contributor license agreements.  See the NOTICE file
  distributed with this work for additional information
  regarding copyright ownership.  The ASF licenses this file
  to you under the Apache License, Version 2.0 (the
  "License"); you may not use this file except in compliance
  with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

/*
  THIS FILE IS AUTOGENERATED, DO NOT EDIT

  Regenerate this file with the 'generate-cripts-config' target. Remember
  to run the format target on the generated file as well.
*/
#pragma once

#include "cripts/ConfigsBase.hpp"

namespace cripts
{
""")


def print_footer():
    print("""

} // namespace cripts
""")


def print_class(tree, cur="", indent=0):
    firstInstance = True
    for k in tree.keys():
        if isinstance(tree[k], dict):
            if indent > 0:
                indentp("private:", indent - 1)
                if cur == "proxy":
                    indentp("friend class cripts::Context; // Needed to set the state", indent)
                    print()
            indentp("class {}".format(k.title()), indent)
            indentp("{", indent)
            print_class(tree[k], k, indent + 1)
        else:
            if firstInstance:
                indentp("public:", indent - 1)
                firstInstance = False
            if tree[k][1] == "TS_RECORDDATATYPE_INT":
                indentp("cripts::IntConfig {}{{\"{}\"}};".format(k, tree[k][2]), indent)
            elif tree[k][1] == "TS_RECORDDATATYPE_FLOAT":
                indentp("cripts::FloatConfig {}{{\"{}\"}};".format(k, tree[k][2]), indent)
            elif tree[k][1] == "TS_RECORDDATATYPE_STRING":
                indentp("cripts::StringConfig {}{{\"{}\"}};".format(k, tree[k][2]), indent)
            else:
                print("The source file has a bad configuration data type: {}".format(tree[k][1]))
    if cur:
        indentp("}}; // End class {}".format(cur.title()), indent - 1)
        print()
        if cur != "proxy":
            indentp("public:", indent - 2)
            indentp("{} {};".format(cur.title(), cur), indent - 1)
            print()


lines = list(fileinput.input())
i = 0
elements = []

while i < len(lines):
    line = lines[i].strip().rstrip()
    pos1 = line.find("proxy.config")
    if pos1 >= 0:
        pos2 = line.find("TS_CONFIG")
        if pos2 < 0:  # Split line
            line += lines[i + 1].strip().rstrip()
        (conf, enum, type) = line.split(",", 2)
        conf = conf.strip(" {\"").rstrip("\" ")
        enum = enum.strip(" {")
        type = type.strip(" ").rstrip(",}); ")
        elements.append((conf, enum, type))
    i += 1

tree = {}
elements.sort()
for elem in elements:
    cur = tree
    name = elem[0]
    parts = elem[0].split(".")
    size = len(parts) - 1
    for ix, part in enumerate(parts):
        sub = cur.get(part)
        if sub is None:
            if ix == size:
                cur[part] = (elem[1], elem[2], name)
            else:
                cur[part] = {}
            cur = cur[part]
        else:
            cur = sub

print_header()
print_class(tree)
print_footer()
# pp = pprint.PrettyPrinter(indent=4)
# pp.pprint(tree)



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from jsonrpc import Notification, Request, Response

Test.Summary = 'Basic records test. Testing the new records.yaml logic and making sure it works as expected.'

ts = Test.MakeATSProcess("ts")

ts.Disk.records_config.update(
    '''
    diags:
      debug:
        enabled: 1
        tags: rpc|rec
    ssl:
      client:
        cert:
          filename: null
          filenamee: some.txt
          filenam: some2.txt
    dns:
      nameservers: null
    test:
      not_registered:

        # we expect this to be parsed and added to the records but inform that
        # the record field is not registered.
        field1: !!int 1

        # we expect this to fail and report while parsing as the field is not
        # registered nor have a valid type
        # field2: 0 # we cant test this now as traffic_layout will output the error and setup.cli.ext will fail
        # to encode json
        field2: 0

    cache:
      # test multipliers with and without tag
      ram_cache:
        size: 30G
    ''')

# 0 - We want to make sure that the unregistered records are still being detected.
tr = Test.AddTestRun("Load unregistered records")
tr.Processes.Default.Command = 'echo 1'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.StartBefore(ts)

var1 = 'proxy.config.test.not_registered.field1'
ts.Disk.diags_log.Content = Testers.ContainsExpression(f"Unrecognized configuration value '{var1}'", "Field should be ignored")
var2 = 'proxy.config.test.not_registered.field2'
ts.Disk.diags_log.Content += Testers.ContainsExpression(f"Unrecognized configuration value '{var2}", "Field should be ignored")

ts.Disk.traffic_out.Content += Testers.ContainsExpression(f"Ignoring field 'filenamee'", "Field should be ignored")
ts.Disk.traffic_out.Content += Testers.ContainsExpression(f"Ignoring field 'filenam'", "Field should be ignored")

# 1
tr = Test.AddTestRun("Query unregistered records.")


def check_response(resp: Response):
    if resp.is_error():
        return (False, resp.error_as_str())

    lst = resp.result['recordList']

    if len(lst) != 2:
        return (False, "Should be 2 records only.")

    for el in lst:
        rec = el['record']
        if rec['registered'] != "false":
            return (False, f"{rec['record_name']} should not be registered")

    return (True, "All good")


tr.AddJsonRPCClientRequest(
    ts, Request.admin_lookup_records([{
        "record_name_regex": "proxy.config.test.not_registered.field",
        "rec_types": ["1", "16"]
    }]))

# do our own check on the response.
tr.Processes.Default.Streams.stdout = Testers.CustomJSONRPCResponse(check_response)

# 2
tctl = Test.AddTestRun("Test field with multiplier")
tctl.Processes.Default.Command = 'traffic_ctl config get proxy.config.cache.ram_cache.size'
tctl.Processes.Default.Env = ts.Env
tctl.Processes.Default.ReturnCode = 0
tctl.Processes.Default.Streams.stdout = Testers.ContainsExpression(
    'proxy.config.cache.ram_cache.size: 32212254720', 'Should hold the configured value.')

# The whole idea is to test how ATS handles having multiple docs in the same records.yaml
# file.
# Every subsequent document will overwrite the previous one, this tests are meant to
# exercise that logic.

ts2 = Test.MakeATSProcess("ts2")

# We are adding a new doc at the end of the original one created by the unit test.
ts2.Disk.records_config.append_to_document('''
    diags:
      debug:
        enabled: 0
        tags: rpc|rec
    ''')

# This will append a new doc in the same file, this config value should overwrite
# the previous one.
ts2.Disk.records_config.append_to_document('''
    diags:
      debug:
        tags: filemanager
    ''')

# After all the loading completed we want to have enabled=0 and tags=filemanager

tr = Test.AddTestRun("Start a new ATS instance.")
tr.Processes.Default.Command = 'echo 1'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.StartBefore(ts2)
tr.Processes.StillRunningAfter = ts2

tr2 = Test.AddTestRun("Test multiple docs from the same file")
tr2.Processes.Default.Command = 'traffic_ctl config get proxy.config.diags.debug.enabled proxy.config.diags.debug.tags'
tr2.Processes.Default.Env = ts2.Env
tr2.Processes.Default.ReturnCode = 0

# Make sure it's what we want.
tr2.Processes.Default.Streams.stdout += Testers.ContainsExpression(
    'proxy.config.diags.debug.enabled: 0', 'Config should show debug disabled')
tr2.Processes.Default.Streams.stdout += Testers.ContainsExpression(
    'proxy.config.diags.debug.tags: filemanager', 'Config should show a different tag')

ts2.Disk.records_config.append_to_document(
    '''
    dns:
      resolv_conf: NULL
      nameservers: null
      local_ipv6: Null
    ssl:
      client:
        cert:
          filename: ~
    ''')

tr3 = Test.AddTestRun("test null string")
tr3.Processes.Default.Command = 'traffic_ctl config get  proxy.config.dns.resolv_conf proxy.config.dns.local_ipv6 proxy.config.dns.nameservers proxy.config.ssl.client.cert.filename'
tr3.Processes.Default.Env = ts2.Env
tr3.Processes.Default.ReturnCode = 0

# Make sure it's what we want.
tr3.Processes.Default.Streams.stdout += Testers.ContainsExpression('proxy.config.dns.resolv_conf: null', 'should be set to null')
tr3.Processes.Default.Streams.stdout += Testers.ContainsExpression('proxy.config.dns.nameservers: null', 'should be set to null')
tr3.Processes.Default.Streams.stdout += Testers.ContainsExpression(
    'proxy.config.ssl.client.cert.filename: null', 'should be set to null')
tr3.Processes.Default.Streams.stdout += Testers.ContainsExpression('proxy.config.dns.local_ipv6: null', 'should be set to null')

ts3 = Test.MakeATSProcess("ts3")
ts3.Disk.records_config.update('''
    some_invalid_field_should_not_block_further_docs_from_the_parser_logic: OK
    ''')

ts3.Disk.records_config.append_to_document('''
    diags:
      debug:
        tags: rpc|rec
    ''')

# We want to make sure that any error in any of the docs will not stop ATS from loading
# the rest of the nodes.
tr4 = Test.AddTestRun("test parsing multiple docs with invalid fields.")
tr4.Processes.Default.Command = 'traffic_ctl config get proxy.config.diags.debug.tags'
tr4.Processes.Default.Env = ts3.Env
tr.Processes.Default.StartBefore(ts3)
tr.Processes.StillRunningAfter = ts3
tr4.Processes.Default.ReturnCode = 0

# This record should be parsed after the invalid field. Value should be the one
# configured.
tr4.Processes.Default.Streams.stdout += Testers.ContainsExpression(
    'proxy.config.diags.debug.tags: rpc|rec', 'should be the configured value')



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import hashlib
import hmac

Test.Summary = '''
Test url_sig plugin
'''

Test.ContinueOnFail = True

# Skip if plugins not present.
Test.SkipUnless(Condition.PluginExists('url_sig.so'))

# Set up to check the output after the tests have run.
#
url_sig_log_id = Test.Disk.File("url_sig_short.log")
url_sig_log_id.Content = "url_sig.gold"

server = Test.MakeOriginServer("server")

request_header = {
    "headers": "GET /foo/abcde/qrstuvwxyz HTTP/1.1\r\nHost: just.any.thing\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
# expected response from the origin server
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
# add response to the server dictionary
server.addResponse("sessionfile.log", request_header, response_header)

# Define default ATS. Disable the cache to make sure each request is forwarded
# to the origin server.
ts = Test.MakeATSProcess("ts", enable_tls=True, enable_cache=False)

ts.addDefaultSSLFiles()

ts.Disk.records_config.update(
    {
        # 'proxy.config.diags.debug.enabled': 1,
        # 'proxy.config.diags.debug.tags': 'http|url_sig',
        'proxy.config.proxy_name': 'Poxy_Proxy',  # This will be the server name.
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
    })

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

# Use unchanged incoming URL.
#
ts.Setup.Copy("url_sig.config", ts.Variables.CONFIGDIR)
ts.Disk.remap_config.AddLine(
    f'map http://one.two.three/ http://127.0.0.1:{server.Variables.Port}/' + ' @plugin=url_sig.so @pparam=url_sig.config')

# Use unchanged incoming HTTPS URL.
#
ts.Disk.remap_config.AddLine(
    f'map https://one.two.three/ http://127.0.0.1:{server.Variables.Port}/' + ' @plugin=url_sig.so @pparam=url_sig.config')

# Use pristine URL, incoming URL unchanged.
#
ts.Disk.remap_config.AddLine(
    f'map http://four.five.six/ http://127.0.0.1:{server.Variables.Port}/' +
    ' @plugin=url_sig.so @pparam=url_sig.config @pparam=pristineurl')

# Use pristine URL, incoming URL changed.
#
ts.Disk.remap_config.AddLine(
    f'map http://seven.eight.nine/ http://127.0.0.1:{server.Variables.Port}' +
    ' @plugin=url_sig.so @pparam=url_sig.config @pparam=PristineUrl')

# Use config with all settings set
#
ts.Setup.Copy("url_sig.all.config", ts.Variables.CONFIGDIR)
ts.Disk.remap_config.AddLine(
    f'map http://ten.eleven.twelve/ http://127.0.0.1:{server.Variables.Port}/' + ' @plugin=url_sig.so @pparam=url_sig.all.config')

# Validation failure tests.

LogTee = f" 2>&1 | grep '^<' | tee -a {Test.RunDirectory}/url_sig_long.log"

# Bad client / MD5 / P=101 / URL pristine / URL altered.
#
tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.StartBefore(server, ready=When.PortOpen(server.Variables.Port))
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://seven.eight.nine/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.2&E=33046620008&A=2&K=13&P=101&S=d1f352d4f1d931ad2f441013402d93f8'" + LogTee)

# With client / MD5 / P=010 / URL pristine / URL altered -- Expired.
#
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://seven.eight.nine/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.1&E=1&A=2&K=13&P=010&S=f237aad1fa010234d7bf8108a0e36387'" + LogTee)

# With client / No algorithm / P=101 / URL pristine / URL altered.
#
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://seven.eight.nine/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.1&E=33046620008&K=13&P=101&S=d1f352d4f1d931ad2f441013402d93f8'" + LogTee)

# With client / Bad algorithm / P=101 / URL pristine / URL altered.
#
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://seven.eight.nine/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.1&E=33046620008&A=3&K=13&P=101&S=d1f352d4f1d931ad2f441013402d93f8'" + LogTee)

# With client / MD5 / No parts / URL pristine / URL altered.
#
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://seven.eight.nine/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.1&E=33046620008&A=2&K=13&S=d1f352d4f1d931ad2f441013402d93f8'" + LogTee)

# With client / MD5 / P=10 (bad) / URL pristine / URL altered.
#
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://seven.eight.nine/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.1&E=33046620008&A=2&K=13&P=10&S=d1f352d4f1d931ad2f441013402d93f8'" + LogTee)

# With client / MD5 / P=101 / URL pristine / URL altered -- No signature.
#
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://seven.eight.nine/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.1&E=33046620008&A=2&K=13&P=101'" + LogTee)

# With client / MD5 / P=101 / URL pristine / URL altered  -- Bad signature.
#
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://seven.eight.nine/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.1&E=33046620008&A=2&K=13&P=101&S=d1f452d4f1d931ad2f441013402d93f8'" + LogTee)

# With client / MD5 / P=101 / URL pristine / URL altered -- Spurious &.
#
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://seven.eight.nine/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.1&E=33046620008&A=2&&K=13&P=101&S=d1f352d4f1d931ad2f441013402d93f8#'" + LogTee)

# Success tests.

# With client / SHA1 / P=1 / URL pristine / URL not altered.
#
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://four.five.six/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.1&E=33046618556&A=1&K=15&P=1&S=f4103561a23adab7723a89b9831d77e0afb61d92'" + LogTee)

# No client / MD5 / P=1 / URL pristine / URL altered.
#
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://seven.eight.nine/" +
    "foo/abcde/qrstuvwxyz?E=33046618586&A=2&K=0&P=1&S=0364efa28afe345544596705b92d20ac'" + LogTee)

# With client / MD5 / P=010 / URL pristine / URL altered.
#
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://seven.eight.nine/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.1&E=33046619717&A=2&K=13&P=010&S=f237aad1fa010234d7bf8108a0e36387'" + LogTee)

# With client / MD5 / P=101 / URL pristine / URL altered.
#
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://seven.eight.nine/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.1&E=33046620008&A=2&K=13&P=101&S=d1f352d4f1d931ad2f441013402d93f8'" + LogTee)


def sign(payload, key):
    secret = bytes(key, 'utf-8')
    data = bytes(payload, 'utf-8')
    md = bytes(hmac.new(secret, data, digestmod=hashlib.sha1).digest().hex(), 'utf-8')
    return md.decode("utf-8")


# No client / SHA1 / P=1 / URL not pristine / URL not altered.
#
path = "foo/abcde/qrstuvwxyz?E=33046618506&A=1&K=7&P=1&S="
to_sign = f"127.0.0.1:{server.Variables.Port}/{path}"
url = "http://one.two.three/" + path + sign(to_sign, "dqsgopTSM_doT6iAysasQVUKaPykyb6e")

tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} '{url}'" + LogTee)

# With client / MD5 / P=101 / URL pristine / URL altered.
# uses url_type pristine in config
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://ten.eleven.twelve/" +
    "foo/abcde/qrstuvwxyz?C=127.0.0.1&E=33046620008&A=2&K=13&P=101&S=586ef8e808caeeea025c525c89ff2638'" + LogTee)

# Single fail test - check for bad path param inserted
# With client / MD5 / P=101 / URL pristine / URL altered. Bad Path Param
# uses url_type pristine in config
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://ten.eleven.twelve/" +
    "foo/abcde/qrstuvwxyz;badparam=true?C=127.0.0.1&E=33046620008&A=2&K=13&P=101&S=586ef8e808caeeea025c525c89ff2638'" + LogTee)

# Success
# With client / SHA1 / P=1 / URL pristine / URL altered. Base64 Encoded Path Param
tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --proxy http://127.0.0.1:{ts.Variables.port} 'http://ten.eleven.twelve/" +
    "foo/abcde;urlsig=Qz0xMjcuMC4wLjE7RT0zMzA0NjYyMDAwODtBPTI7Sz0xMztQPTEwMTtTPTA1MDllZjljY2VlNjUxZWQ1OTQxM2MyZjE3YmVhODZh/qrstuvwxyz'"
    + LogTee)

# Success
# This test must be last since it converts from the long to the short log output
# No client / SHA1 / P=1 / URL not pristine / URL not altered -- HTTPS.
#
path = "foo/abcde/qrstuvwxyz?E=33046618506&A=1&K=7&P=1&S="
to_sign = f"127.0.0.1:{server.Variables.Port}/{path}"
url = f"https://127.0.0.1:{ts.Variables.ssl_port}/{path}" + sign(to_sign, "dqsgopTSM_doT6iAysasQVUKaPykyb6e")

tr = Test.AddTestRun()
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Command = (
    f"curl --verbose --http1.1 --insecure --header 'Host: one.two.three' '{url}'" + LogTee +
    " ; grep -F -e '< HTTP' -e Authorization {0}/url_sig_long.log > {0}/url_sig_short.log ".format(ts.RunDirectory))

# Overriding the built in ERROR check since we expect some ERROR messages
ts.Disk.diags_log.Content = Testers.ContainsExpression("ERROR", "Some tests are failure tests")
ts.Disk.diags_log.Content += Testers.ExcludesExpression("Error parsing", "Verify that we can accept long comment lines")



#!/usr/bin/env python3
"""Common logic between the ad hoc client and server."""

#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import socket


def wait_for_headers_complete(sock: socket.socket) -> bytes:
    """Wait for the headers to be complete.

    :param sock: The socket to read from.
    :returns: The bytes read off the socket.
    """
    headers = b""
    while True:
        data = sock.recv(1024)
        if not data:
            print("Socket closed.")
            break
        print(f'Received:\n{data}')
        headers += data
        if b"\r\n\r\n" in headers:
            break
    return headers


def determine_outstanding_bytes_to_read(read_bytes: bytes) -> int:
    """Determine how many more bytes to read from the headers.

    This parses the Content-Length header to determine how many more bytes to
    read.

    :param read_bytes: The bytes read so far.
    :returns: The number of bytes to read, or -1 if it is chunked encoded.
    """
    headers = read_bytes.decode("utf-8").split("\r\n")
    content_length_value = None
    for header in headers:
        if header.lower().startswith("content-length:"):
            content_length_value = int(header.split(":")[1].strip())
        elif header.lower().startswith("transfer-encoding: chunked"):
            return -1
    if content_length_value is None:
        raise ValueError("No Content-Length header found.")

    end_of_headers = read_bytes.find(b"\r\n\r\n")
    if end_of_headers == -1:
        raise ValueError("No end of headers found.")

    end_of_headers += 4
    return content_length_value - (len(read_bytes) - end_of_headers)


def drain_socket(sock: socket.socket, previously_read_data: bytes, num_bytes_to_drain: int) -> None:
    """Read the rest of the request.

    :param sock: The socket to drain.
    :param num_bytes_to_drain: The number of bytes to drain. If -1, then drain
    bytes until the final zero-length chunk is read.
    """

    read_data = previously_read_data
    num_bytes_drained = 0
    while True:
        if num_bytes_to_drain > 0:
            if num_bytes_drained >= num_bytes_to_drain:
                break
        elif b'0\r\n\r\n' == read_data[-5:]:
            print("Found end of chunked data.")
            break

        data = sock.recv(1024)
        print(f'Received:\n{data}')
        if not data:
            print("Socket closed.")
            break
        num_bytes_drained += len(data)
        read_data += data



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import sys

Test.Summary = '''
Test tunneling based on SNI
'''

# Define default ATS
ts = Test.MakeATSProcess("ts", enable_tls=True, enable_proxy_protocol=True)
server_foo = Test.MakeOriginServer("server_foo", ssl=True)
server_bar = Test.MakeOriginServer("server_bar", ssl=True)
server2 = Test.MakeOriginServer("server2")
# The following server will listen on a port that is not in the connect_ports
# list.
server_forbidden = Test.MakeOriginServer("forbidden", ssl=True)
dns = Test.MakeDNServer("dns")

request_foo_header = {"headers": "GET / HTTP/1.1\r\nHost: foo.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
request_bar_header = {"headers": "GET / HTTP/1.1\r\nHost: bar.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
request_pp_header = {
    "headers": "GET /proxy_protocol HTTP/1.1\r\nHost: proxy.protocol.port.com\r\n\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_foo_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": "foo ok"}
response_bar_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": "bar ok"}
response_pp_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": "pp ok"}
server_foo.addResponse("sessionlog.json", request_foo_header, response_foo_header)
server_foo.addResponse("sessionlog.json", request_pp_header, response_pp_header)
server_bar.addResponse("sessionlog.json", request_bar_header, response_bar_header)
server_forbidden.addResponse("sessionlog.json", request_pp_header, response_pp_header)

# add ssl materials like key, certificates for the server
ts.addSSLfile("ssl/signed-foo.pem")
ts.addSSLfile("ssl/signed-foo.key")
ts.addSSLfile("ssl/signed-bar.pem")
ts.addSSLfile("ssl/signed-bar.key")
ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")
ts.addSSLfile("ssl/signer.pem")
ts.addSSLfile("ssl/signer.key")

dns.addRecords(records={"localhost": ["127.0.0.1"]})
dns.addRecords(records={"one.testmatch": ["127.0.0.1"]})
dns.addRecords(records={"two.example.one": ["127.0.0.1"]})
dns.addRecords(records={"backend.incoming.port.com": ["127.0.0.1"]})
dns.addRecords(records={"backend.proxy.protocol.port.com": ["127.0.0.1"]})
dns.addRecords(records={"backend.wildcard.with.incoming.port.com": ["127.0.0.1"]})
dns.addRecords(records={"backend.wildcard.with.proxy.protocol.port.com": ["127.0.0.1"]})
# Need no remap rules.  Everything should be processed by sni

# Make sure the TS server certs are different from the origin certs
ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=signed-foo.pem ssl_key_name=signed-foo.key')

# Case 1, global config policy=permissive properties=signature
#         override for foo.com policy=enforced properties=all
ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.http.connect_ports':
            '{0} {1} {2}'.format(ts.Variables.ssl_port, server_foo.Variables.SSL_Port, server_bar.Variables.SSL_Port),
        'proxy.config.ssl.client.CA.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.client.CA.cert.filename': 'signer.pem',
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.url_remap.pristine_host_hdr': 1,
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http|ssl_sni|proxyprotocol',
        'proxy.config.dns.nameservers': '127.0.0.1:{0}'.format(dns.Variables.Port),
        'proxy.config.dns.resolv_conf': 'NULL'
    })

# foo.com should not terminate.  Just tunnel to server_foo
# bar.com should terminate.  Forward its tcp stream to server_bar
# empty SNI should tunnel to server_bar
ts.Disk.sni_yaml.AddLines(
    [
        'sni:',
        '- fqdn: foo.com',
        "  tunnel_route: localhost:{0}".format(server_foo.Variables.SSL_Port),
        "- fqdn: '*.bar.com'",
        "  tunnel_route: localhost:{0}".format(server_foo.Variables.SSL_Port),
        "- fqdn: '*.match.com'",
        "  tunnel_route: $1.testmatch:{0}".format(server_foo.Variables.SSL_Port),
        "- fqdn: '*.ok.two.com'",
        "  tunnel_route: two.example.$1:{0}".format(server_foo.Variables.SSL_Port),
        "- fqdn: ''",  # No SNI sent
        "  tunnel_route: localhost:{0}".format(server_bar.Variables.SSL_Port),
        "- fqdn: 'incoming.port.com'",
        "  tunnel_route: backend.incoming.port.com:{inbound_local_port}",
        "- fqdn: 'proxy.protocol.port.com'",
        "  tunnel_route: backend.proxy.protocol.port.com:{proxy_protocol_port}",
        "- fqdn: '*.backend.incoming.port.com'",
        "  tunnel_route: backend.$1.incoming.port.com:{inbound_local_port}",
        "- fqdn: '*.with.incoming.port.com'",
        "  tunnel_route: backend.$1.with.incoming.port.com:{inbound_local_port}",
        "- fqdn: '*.with.proxy.protocol.port.com'",
        "  tunnel_route: backend.$1.with.proxy.protocol.port.com:{proxy_protocol_port}",
    ])

tr = Test.AddTestRun("foo.com Tunnel-test")
tr.Processes.Default.Command = "curl -v --resolve 'foo.com:{0}:127.0.0.1' -k  https://foo.com:{0}".format(ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.Processes.Default.StartBefore(server_foo)
tr.Processes.Default.StartBefore(server_bar)
tr.Processes.Default.StartBefore(dns)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.StillRunningAfter = ts
tr.StillRunningAfter = server_foo
tr.StillRunningAfter = server_bar
tr.StillRunningAfter = dns
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression(
    "Not Found on Accelerato", "Should not try to remap on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("CN=foo.com", "Should not TLS terminate on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/1.1 200 OK", "Should get a successful response")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("ATS", "Do not terminate on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("foo ok", "Should get a response from bar")

tr = Test.AddTestRun("bob.bar.com Tunnel-test")
tr.Processes.Default.Command = "curl -v --resolve 'bob.bar.com:{0}:127.0.0.1' -k  https://bob.bar.com:{0}".format(
    ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression(
    "Not Found on Accelerato", "Should not try to remap on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("CN=foo.com", "Should not TLS terminate on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/1.1 200 OK", "Should get a successful response")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("ATS", "Do not terminate on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("foo ok", "Should get a response from bar")

tr = Test.AddTestRun("bar.com no Tunnel-test")
tr.Processes.Default.Command = "curl -v --resolve 'bar.com:{0}:127.0.0.1' -k  https://bar.com:{0}".format(ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("Not Found on Accelerato", "Terminates on on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("ATS", "Terminate on Traffic Server")

tr = Test.AddTestRun("no SNI Tunnel-test")
tr.Processes.Default.Command = "curl -v -k  https://127.0.0.1:{0}".format(ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression(
    "Not Found on Accelerato", "Should not try to remap on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/1.1 200 OK", "Should get a successful response")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("ATS", "Do not terminate on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("bar ok", "Should get a response from bar")

tr = Test.AddTestRun("one.match.com Tunnel-test")
tr.Processes.Default.Command = "curl -vvv --resolve 'one.match.com:{0}:127.0.0.1' -k  https://one.match.com:{0}".format(
    ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression(
    "Not Found on Accelerato", "Should not try to remap on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("CN=foo.com", "Should not TLS terminate on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/1.1 200 OK", "Should get a successful response")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("ATS", "Do not terminate on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("foo ok", "Should get a response from tm")

tr = Test.AddTestRun("one.ok.two.com Tunnel-test")
tr.Processes.Default.Command = "curl -vvv --resolve 'one.ok.two.com:{0}:127.0.0.1' -k  https://one.ok.two.com:{0}".format(
    ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression(
    "Not Found on Accelerato", "Should not try to remap on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("CN=foo.com", "Should not TLS terminate on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/1.1 200 OK", "Should get a successful response")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("ATS", "Do not terminate on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("foo ok", "Should get a response from tm")

tr = Test.AddTestRun("test {inbound_local_port}")
tr.Processes.Default.Command = "curl -vvv --resolve 'incoming.port.com:{0}:127.0.0.1' -k  https://incoming.port.com:{0}".format(
    ts.Variables.ssl_port)
# The tunnel connecting to the outgoing port which is the same as the incoming
# port (per the `inbound_local_port` configuration) will result in ATS
# connecting back to itself. This will result in a connection close and a
# non-zero return code from curl. In production, the server will listen on the
# same port as ATS but have a different IP.
tr.ReturnCode = 35
tr.StillRunningAfter = ts
ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    f"CONNECT tunnel://backend.incoming.port.com:{ts.Variables.ssl_port} HTTP/1.1", "Verify a CONNECT request is handled")
ts.Disk.traffic_out.Content += Testers.ContainsExpression("HTTP/1.1 400 Cycle Detected", "The loop should be detected")

tr = Test.AddTestRun("test {proxy_protocol_port}")
tr.Setup.Copy('proxy_protocol_client.py')
tr.Processes.Default.Command = (
    f'{sys.executable} proxy_protocol_client.py '
    f'127.0.0.1 {ts.Variables.proxy_protocol_ssl_port} proxy.protocol.port.com '
    f'127.0.0.1 127.0.0.1 60123 {server_foo.Variables.SSL_Port} '
    f'2 --https')
tr.ReturnCode = 0
tr.TimeOut = 5
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/1.1 200 OK", "Verify a successful response is received")

# This test run causes an exception in proxy_protocol_client.py
#
tr = Test.AddTestRun("test proxy_protocol_port - not in connect_ports")
tr.Processes.Default.StartBefore(server_forbidden)
tr.Setup.Copy('proxy_protocol_client.py')
# Note that server_forbidden is listing on a port that is not in the
# connect_ports list. Therefore the tunnel should be rejected.
rejected_port = server_forbidden.Variables.SSL_Port
tr.Processes.Default.Command = (
    f'{sys.executable} proxy_protocol_client.py '
    f'127.0.0.1 {ts.Variables.proxy_protocol_ssl_port} proxy.protocol.port.com '
    f'127.0.0.1 127.0.0.1 60123 {rejected_port} '
    f'2 --https')
tr.ReturnCode = 1
tr.TimeOut = 5
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ContainsExpression("ssl.SSL.*Error:.*EOF", "Verify a the handshake failed")
ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    f"Rejected a tunnel to port {rejected_port} not in connect_ports", "Verify the tunnel was rejected")

tr = Test.AddTestRun("test wildcard with inbound_local_port")
tr.Processes.Default.Command = "curl -vvv --resolve 'wildcard.with.incoming.port.com:{0}:127.0.0.1' -k  https://wildcard.with.incoming.port.com:{0}".format(
    ts.Variables.ssl_port)

# See the inbound_local_port test above for the explanation of the return code.
tr.ReturnCode = 35
tr.StillRunningAfter = ts
ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    rf"Destination now is \[backend.wildcard.with.incoming.port.com:{ts.Variables.ssl_port}\]",
    "Verify the tunnel destination is expanded correctly.")
ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    f"CONNECT tunnel://backend.wildcard.with.incoming.port.com:{ts.Variables.ssl_port} HTTP/1.1",
    "Verify a CONNECT request is handled")
ts.Disk.traffic_out.Content += Testers.ContainsExpression("HTTP/1.1 400 Cycle Detected", "The loop should be detected")

tr = Test.AddTestRun("test wildcard with proxy_protocol_port")
tr.Setup.Copy('proxy_protocol_client.py')
tr.Processes.Default.Command = (
    f'{sys.executable} proxy_protocol_client.py '
    f'127.0.0.1 {ts.Variables.proxy_protocol_ssl_port} wildcard.with.proxy.protocol.port.com '
    f'127.0.0.1 127.0.0.1 60123 {server_foo.Variables.SSL_Port} '
    f'2 --https')
tr.ReturnCode = 0
tr.TimeOut = 5
tr.StillRunningAfter = ts
ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    rf"Destination now is \[backend.wildcard.with.proxy.protocol.port.com:{server_foo.Variables.SSL_Port}\]",
    "Verify the tunnel destination is expanded correctly.")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/1.1 200 OK", "Verify a successful response is received")

# Update sni file and reload
tr = Test.AddTestRun("Update config files")
# Update the SNI config
snipath = ts.Disk.sni_yaml.AbsPath
recordspath = ts.Disk.records_config.AbsPath
tr.Disk.File(snipath, id="sni_yaml", typename="ats:config"),
tr.Disk.sni_yaml.AddLines([
    'sni:',
    '- fqdn: bar.com',
    '  tunnel_route: localhost:{0}'.format(server_bar.Variables.SSL_Port),
])
tr.StillRunningAfter = ts
tr.StillRunningAfter = server_foo
tr.StillRunningAfter = server_bar
tr.Processes.Default.Env = ts.Env
tr.Processes.Default.Command = 'echo Updated configs'
tr.Processes.Default.ReturnCode = 0

trreload = Test.AddTestRun("Reload config")
trreload.StillRunningAfter = ts
trreload.StillRunningAfter = server_foo
trreload.StillRunningAfter = server_bar
trreload.Processes.Default.Command = 'traffic_ctl config reload'
# Need to copy over the environment so traffic_ctl knows where to find the unix domain socket
trreload.Processes.Default.Env = ts.Env
trreload.Processes.Default.ReturnCode = 0

# Should terminate on traffic_server (not tunnel)
tr = Test.AddTestRun("foo.com no Tunnel-test")
tr.StillRunningAfter = ts
# Wait for the reload to complete by running the sni_reload_done test
tr.Processes.Default.StartBefore(server2, ready=When.FileContains(ts.Disk.diags_log.Name, 'sni.yaml finished loading', 2))
tr.Processes.Default.Command = "curl -v --resolve 'foo.com:{0}:127.0.0.1' -k  https://foo.com:{0}".format(ts.Variables.ssl_port)
tr.Processes.Default.Streams.All += Testers.ContainsExpression("Not Found on Accelerato", "Terminates on on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("ATS", "Terminate on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr.TimeOut = 30

# Should tunnel to server_bar
tr = Test.AddTestRun("bar.com  Tunnel-test")
tr.Processes.Default.Command = "curl -v --resolve 'bar.com:{0}:127.0.0.1' -k  https://bar.com:{0}".format(ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.StillRunningAfter = ts
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("Not Found on Accelerato", "Terminates on on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("ATS", "Terminate on Traffic Server")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("bar ok", "Should get a response from bar")

tr = Test.AddTestRun("Test Metrics")
tr.Processes.Default.Command = (
    f"{Test.Variables.AtsTestToolsDir}/stdout_wait" + " 'traffic_ctl metric get" +
    " proxy.process.http.total_incoming_connections" + " proxy.process.http.total_client_connections" +
    " proxy.process.http.total_client_connections_ipv4" + " proxy.process.http.total_client_connections_ipv6" +
    " proxy.process.http.total_server_connections" + " proxy.process.http2.total_client_connections" +
    " proxy.process.http.connect_requests" + " proxy.process.tunnel.total_client_connections_blind_tcp" +
    " proxy.process.tunnel.current_client_connections_blind_tcp" + " proxy.process.tunnel.total_server_connections_blind_tcp" +
    " proxy.process.tunnel.current_server_connections_blind_tcp" + " proxy.process.tunnel.total_client_connections_tls_tunnel" +
    " proxy.process.tunnel.current_client_connections_tls_tunnel" + " proxy.process.tunnel.total_client_connections_tls_forward" +
    " proxy.process.tunnel.current_client_connections_tls_forward" +
    " proxy.process.tunnel.total_client_connections_tls_partial_blind" +
    " proxy.process.tunnel.current_client_connections_tls_partial_blind" +
    " proxy.process.tunnel.total_client_connections_tls_http" + " proxy.process.tunnel.current_client_connections_tls_http" +
    " proxy.process.tunnel.total_server_connections_tls" + " proxy.process.tunnel.current_server_connections_tls'" +
    f" {Test.TestDirectory}/gold/tls-tunnel-metrics.gold")
# Need to copy over the environment so traffic_ctl knows where to find the unix domain socket
tr.Processes.Default.Env = ts.Env
tr.Processes.Default.ReturnCode = 0
tr.StillRunningAfter = ts



# @file
#
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
#  Copyright 2021, Verizon Media
#

import os.path

Test.Summary = '''
Production use case: Manipulate specific query parameters.
'''

Test.SkipUnless(Condition.PluginExists("txn_box.so"))

replay_file = "query-delete.replay.yaml"

tr = Test.TxnBoxTestAndRun(
    "Query Delete with RXP",
    replay_file,
    config_path='Auto',
    config_key="meta.txn-box.alpha",
    verifier_client_args="--verbose info")

ts = tr.Variables.TS

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'txn_box',
        'proxy.config.http.cache.http': 0
    })
